@@ -32,7 +32,7 @@
 #include <goffice/app/go-plugin-loader.h>
 #include <goffice/app/module-plugin-defs.h>
 #include <gsf/gsf-impl-utils.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <glib/gstdio.h>
 #include <stdlib.h>
 #include <glib.h>
@@ -110,7 +110,7 @@ gplp_load_base (GOPluginLoader *loader, ErrorInfo **ret_error)
 	gchar *full_module_file_name = NULL;
 	FILE *f;
 	GOPlugin *plugin = go_plugin_loader_get_plugin (loader);
-	ErrorInfo *open_error;
+	ErrorInfo *open_error = NULL;
 	PyObject *modules, *main_module, *main_module_dict;
 
 	GO_INIT_RET_ERROR_INFO (ret_error);
@@ -451,7 +451,7 @@ gplp_loader_data_fngroup_free (ServiceLoaderDataFunctionGroup *loader_data)
 }
 
 static GnmValue *
-call_python_function_args (FunctionEvalInfo *ei, GnmValue const * const *args)
+call_python_function_args (GnmFuncEvalInfo *ei, GnmValue const * const *args)
 {
 	GOPluginService *service;
 	ServiceLoaderDataFunctionGroup *loader_data;
@@ -481,8 +481,8 @@ call_python_function_args (FunctionEvalInfo *ei, GnmValue const * const *args)
 }
 
 static GnmValue *
-call_python_function_nodes (FunctionEvalInfo *ei,
-			    int argc, const GnmExprConstPtr *argv)
+call_python_function_nodes (GnmFuncEvalInfo *ei,
+			    int argc, GnmExprConstPtr const *argv)
 {
 	GOPluginService *service;
 	ServiceLoaderDataFunctionGroup *loader_data;
