@@ -27,13 +27,12 @@
 #include "commands.h"
 
 #include <gnumeric-gconf.h>
-#include <gsf/gsf-impl-utils.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtkiconfactory.h>
-#include <gtk/gtkselection.h>
-#include <gtk/gtkicontheme.h>
-#include <glib/gi18n.h>
+#include <goffice/app/go-doc.h>
 #include <goffice/utils/go-glib-extras.h>
+#include <goffice/utils/go-file.h>
+#include <gsf/gsf-impl-utils.h>
+#include <gtk/gtk.h>
+#include <glib/gi18n-lib.h>
 
 #define GNM_APP(o)		(G_TYPE_CHECK_INSTANCE_CAST((o), GNM_APP_TYPE, GnmApp))
 #define GNM_APP_CLASS(k)	(G_TYPE_CHECK_CLASS_CAST((k),	 GNM_APP_TYPE, GnmAppClass))
@@ -65,9 +64,6 @@ struct _GnmApp {
 	GnmCellRegion	*clipboard_copied_contents;
 	GnmRange	*clipboard_cut_range;
 
-	/* History for file menu */
-	GSList           *history_list;
-
 	/* Others */
 	GtkWidget       *pref_dialog;
 
@@ -94,44 +90,11 @@ gnm_app_get_app (void)
 	return G_OBJECT (app);
 }
 
-static void
-add_icon (GtkIconFactory *factory,
-	  guchar const   *scalable_data,
-	  guchar const   *sized_data,
-	  gchar const    *stock_id)
-{
-	GtkIconSet *set = gtk_icon_set_new ();
-	GtkIconSource *src = gtk_icon_source_new ();
-
-	if (scalable_data != NULL) {
-		gtk_icon_source_set_size_wildcarded (src, TRUE);
-		gtk_icon_source_set_pixbuf (src,
-			gdk_pixbuf_new_from_inline (-1, scalable_data, FALSE, NULL));
-		gtk_icon_set_add_source (set, src);	/* copies the src */
-	}
-
-	/*
-	 * For now, don't register a fixed-sized icon as doing so without
-	 * catching style changes kills things like bug 302902.
-	 */
-	if (scalable_data == NULL && sized_data != NULL) {
-		gtk_icon_source_set_size (src, GTK_ICON_SIZE_MENU);
-		gtk_icon_source_set_size_wildcarded (src, FALSE);
-		gtk_icon_source_set_pixbuf (src,
-			gdk_pixbuf_new_from_inline (-1, sized_data, FALSE, NULL));
-		gtk_icon_set_add_source (set, src);	/* copies the src */
-	}
-
-	gtk_icon_factory_add (factory, stock_id, set);	/* keeps reference to set */
-	gtk_icon_set_unref (set);
-	gtk_icon_source_free (src);
-}
-
 /**
- * gnm_app_workbook_list_remove :
+ * gnm_app_workbook_list_add :
  * @wb :
  *
- * Remove @wb from the application's list of workbooks.
+ * Add @wb to the application's list of workbooks.
  **/
 void
 gnm_app_workbook_list_add (Workbook *wb)
@@ -141,7 +104,7 @@ gnm_app_workbook_list_add (Workbook *wb)
 
 	app->workbook_list = g_list_prepend (app->workbook_list, wb);
 	g_signal_connect (G_OBJECT (wb),
-		"filename_changed",
+		"notify::uri",
 		G_CALLBACK (_gnm_app_flag_windows_changed), NULL);
 	_gnm_app_flag_windows_changed ();
 	g_signal_emit (G_OBJECT (app), signals [WORKBOOK_ADDED], 0, wb);
@@ -209,6 +172,16 @@ gnm_app_clipboard_clear (gboolean drop_selection)
 	}
 }
 
+void
+gnm_app_clipboard_invalidate_sheet (Sheet *sheet)
+{
+	/* Clear the cliboard to avoid dangling references to the deleted sheet */
+	if (sheet == gnm_app_clipboard_sheet_get ())
+		gnm_app_clipboard_clear (TRUE);
+	else if (app->clipboard_copied_contents)
+		cellregion_invalidate_sheet (app->clipboard_copied_contents, sheet);
+}
+
 void
 gnm_app_clipboard_unant (void)
 {
@@ -365,17 +338,63 @@ gnm_app_clipboard_area_get (void)
 	return NULL;
 }
 
-struct wb_name_closure {
-	Workbook *wb;
-	char const *name;
-};
-static gboolean
-cb_workbook_name (Workbook * wb, gpointer closure)
+Workbook *
+gnm_app_workbook_get_by_name (char const *name,
+			      char const *ref_uri)
 {
-	struct wb_name_closure *dat = closure;
-	const char *wb_uri = workbook_get_uri (wb);
+	Workbook *wb;
+	char *filename = NULL;
 
-	if (wb_uri && strcmp (wb_uri, dat->name) == 0) {
+	if (name == NULL || *name == 0)
+		return NULL;
+
+	/* Try as URI.  */
+	wb = gnm_app_workbook_get_by_uri (name);
+	if (wb)
+		goto out;
+
+	filename = g_filename_from_utf8 (name, -1, NULL, NULL, NULL);
+
+	/* Try as absolute filename.  */
+	if (filename && g_path_is_absolute (filename)) {
+		char *uri = go_filename_to_uri (filename);
+		if (uri) {
+			wb = gnm_app_workbook_get_by_uri (uri);
+			g_free (uri);
+		}
+		if (wb)
+			goto out;
+	}
+
+	if (filename && ref_uri) {
+		char *rel_uri = go_url_encode (filename, 1);
+		char *uri = go_url_resolve_relative (ref_uri, rel_uri);
+		g_free (rel_uri);
+		if (uri) {
+			wb = gnm_app_workbook_get_by_uri (uri);
+			g_free (uri);
+		}
+		if (wb)
+			goto out;
+	}
+
+ out:
+	g_free (filename);
+	return wb;
+}
+
+struct wb_uri_closure {
+	Workbook *wb;
+	char const *uri;
+};
+
+static gboolean
+cb_workbook_uri (Workbook * wb, gpointer closure)
+{
+	struct wb_uri_closure *dat = closure;
+	char const *wb_uri = go_doc_get_uri (GO_DOC (wb));
+
+	if (wb_uri && strcmp (wb_uri, dat->uri) == 0) {
 		dat->wb = wb;
 		return FALSE;
 	}
@@ -383,14 +402,17 @@ cb_workbook_name (Workbook * wb, gpointer closure)
 }
 
 Workbook *
-gnm_app_workbook_get_by_name (char const * const name)
+gnm_app_workbook_get_by_uri (char const *uri)
 {
-	struct wb_name_closure close;
-	close.wb = NULL;
-	close.name = name;
-	gnm_app_workbook_foreach (&cb_workbook_name, &close);
+	struct wb_uri_closure closure;
 
-	return close.wb;
+	g_return_val_if_fail (uri != NULL, NULL);
+
+	closure.wb = NULL;
+	closure.uri = uri;
+	gnm_app_workbook_foreach (&cb_workbook_uri, &closure);
+
+	return closure.wb;
 }
 
 gboolean
@@ -416,21 +438,25 @@ struct wb_index_closure
 };
 
 static gboolean
-cb_workbook_index (G_GNUC_UNUSED Workbook * wb, gpointer closure)
+cb_workbook_index (Workbook *wb, gpointer closure)
 {
 	struct wb_index_closure *dat = closure;
-	return (--(dat->index) != 0);
+	if (--(dat->index) == 0) {
+		dat->wb = wb;
+		return FALSE;
+	}
+	return TRUE;
 }
 
 Workbook *
 gnm_app_workbook_get_by_index (int i)
 {
-	struct wb_index_closure close;
-	close.wb = NULL;
-	close.index = i;
-	gnm_app_workbook_foreach (&cb_workbook_index, &close);
+	struct wb_index_closure closure;
+	closure.wb = NULL;
+	closure.index = i;
+	gnm_app_workbook_foreach (&cb_workbook_index, &closure);
 
-	return close.wb;
+	return closure.wb;
 }
 
 double
@@ -446,36 +472,106 @@ gnm_app_dpi_to_pixels (void)
 		    gnm_app_prefs->vertical_dpi) / 72.;
 }
 
+/* GtkFileFilter */
+void *
+gnm_app_create_opener_filter (void)
+{
+	GtkFileFilter *filter = gtk_file_filter_new ();
+
+	GList *openers;
+
+	for (openers = go_get_file_openers ();
+	     openers;
+	     openers = openers->next) {
+		GOFileOpener *opener = openers->data;
+		const GSList *mimes = go_file_opener_get_mimes (opener);
+		const GSList *suffixes = go_file_opener_get_suffixes (opener);
+
+		while (mimes) {
+			gtk_file_filter_add_mime_type (filter, mimes->data);
+			mimes = mimes->next;
+		}
+
+		while (suffixes) {
+			char *pattern = g_strconcat ("*.", suffixes->data, NULL);
+			gtk_file_filter_add_pattern (filter, pattern);
+			g_free (pattern);
+			suffixes = suffixes->next;
+		}
+	}
+
+	return filter;
+}
+
+#ifdef HAVE_GTK_RECENT_MANAGER_GET_DEFAULT
+static gint
+compare_mru (GtkRecentInfo *a,
+	     GtkRecentInfo *b)
+{
+  return (gtk_recent_info_get_modified (a) < gtk_recent_info_get_modified (b));
+}
+#endif
+
 /**
  * gnm_app_history_get_list:
  *
  * creating it if necessary.
  *
- * Return value: the list./
+ * Return value: the list, which must be freed along with the strings in it.
  **/
-GSList const *
+GSList *
 gnm_app_history_get_list (gboolean force_reload)
 {
-        gint max_entries;
-	GSList const *ptr;
+#ifdef HAVE_GTK_RECENT_MANAGER_GET_DEFAULT
+	GtkRecentManager *manager = gtk_recent_manager_get_default ();
 	GSList *res = NULL;
+	GList *items, *l;
+	GtkFileFilter *filter = gnm_app_create_opener_filter ();
 
-	g_return_val_if_fail (app != NULL, NULL);
+	items = gtk_recent_manager_get_items (manager);
+	items = g_list_sort (items, (GCompareFunc)compare_mru);
 
-	if (app->history_list != NULL) {
-		if (force_reload) {
-			GSList *tmp = app->history_list;
-			app->history_list = NULL;
-			go_slist_free_custom (tmp, g_free);
-		} else
-			return app->history_list;
+	for (l = items; l; l = l->next) {
+		GtkRecentInfo *ri = l->data;
+		const char *uri = gtk_recent_info_get_uri (ri);
+		gboolean want_it;
+
+		if (gtk_recent_info_has_application (ri, g_get_application_name ())) {
+			want_it = TRUE;
+		} else {
+			GtkFileFilterInfo fi;
+			char *display_name = g_filename_display_basename (uri);
+
+			memset (&fi, 0, sizeof (fi));
+			fi.contains = (GTK_FILE_FILTER_MIME_TYPE |
+				       GTK_FILE_FILTER_URI |
+				       GTK_FILE_FILTER_DISPLAY_NAME);
+			fi.uri = uri;
+			fi.mime_type = gtk_recent_info_get_mime_type (ri);
+			fi.display_name = display_name;
+			want_it = gtk_file_filter_filter (filter, &fi);
+			g_free (display_name);
+		}
+
+		if (want_it) {
+			char *filename = go_filename_from_uri (uri);
+			if (filename && !g_file_test (filename, G_FILE_TEST_EXISTS))
+				want_it = FALSE;
+			g_free (filename);
+		}
+
+		if (want_it)
+			res = g_slist_prepend (res, g_strdup (uri));
 	}
 
-	max_entries = gnm_app_prefs->file_history_max;
-	for (ptr = gnm_app_prefs->file_history_files;
-	     ptr != NULL && max_entries-- > 0 ; ptr = ptr->next)
-		res = g_slist_prepend (res, g_strdup (ptr->data));
-	return app->history_list = g_slist_reverse (res);;
+	go_list_free_custom (items, (GFreeFunc)gtk_recent_info_unref);
+	g_object_ref_sink (filter);
+	g_object_unref (filter);
+
+	return g_slist_reverse (res);
+#else
+	return NULL;
+#endif
 }
 
 /**
@@ -485,46 +581,31 @@ gnm_app_history_get_list (gboolean force_reload)
  * Adds @uri to the application's history of files.
  **/
 void
-gnm_app_history_add (char const *uri)
+gnm_app_history_add (char const *uri, const char *mimetype)
 {
-        gint max_entries;
-	GSList *exists;
-	GSList **ptr;
+#ifdef HAVE_GTK_RECENT_MANAGER_GET_DEFAULT
+	GtkRecentData rd;
+	gboolean retval;
+	GtkRecentManager *manager = gtk_recent_manager_get_default ();
 
-	g_return_if_fail (uri != NULL);
-	g_return_if_fail (app != NULL);
+	memset (&rd, 0, sizeof (rd));
 
-	/* force a reload in case max_entries has changed */
-	gnm_app_history_get_list (TRUE);
-	exists = g_slist_find_custom (app->history_list,
-				      uri, go_str_compare);
+        rd.mime_type =
+		g_strdup (mimetype ? mimetype : "application/octet-stream");
 
-	if (exists != NULL) {
-		/* its already the top of the stack no need to do anything */
-		if (exists == app->history_list)
-			return;
+	rd.app_name = g_strdup (g_get_application_name ());
+	rd.app_exec = g_strjoin (" ", g_get_prgname (), "%u", NULL);
+	rd.groups = NULL;
+	rd.is_private = FALSE;
 
-		/* remove the other instance */
-		g_free (exists->data);
-		app->history_list = g_slist_delete_link (app->history_list, exists);
-	}
+	retval = gtk_recent_manager_add_full (manager, uri, &rd);
 
-	app->history_list = g_slist_prepend (app->history_list, g_strdup (uri));
-
-	/* clip the list if it is too long */
-	max_entries = gnm_app_prefs->file_history_max;
-	ptr = &(app->history_list);
-	while (*ptr != NULL && max_entries-- > 0)
-		ptr = &((*ptr)->next);
-	if (*ptr != NULL) {
-		go_slist_free_custom (*ptr, g_free);
-		*ptr = NULL;
-	}
+	g_free (rd.mime_type);
+	g_free (rd.app_name);
+	g_free (rd.app_exec);
+#endif
 
 	g_object_notify (G_OBJECT (app), "file-history-list");
-	gnm_gconf_set_file_history_files (
-		go_string_slist_copy (app->history_list));
-	go_conf_sync (NULL);
 }
 
 gboolean gnm_app_use_auto_complete	  (void) { return gnm_app_prefs->auto_complete; }
@@ -558,212 +639,11 @@ gnm_app_release_pref_dialog (void)
 		gtk_widget_destroy (app->pref_dialog);
 }
 
-static void
-gnumeric_application_setup_pixbufs (GnmApp *app)
-{
-	static struct {
-		guchar const   *scalable_data;
-		gchar const    *name;
-	} const entry [] = {
-		/* Cursors */
-		{ gnm_cursor_cross, "cursor_cross" },
-		{ gnm_bucket, "bucket" },
-		{ gnm_font, "font" },
-		{ sheet_move_marker, "sheet_move_marker" },
-		/* Patterns */
-		{ gp_125grey, "gp_125grey" },
-		{ gp_25grey, "gp_25grey" },
-		{ gp_50grey, "gp_50grey" },
-		{ gp_625grey, "gp_625grey" },
-		{ gp_75grey, "gp_75grey" },
-		{ gp_bricks, "gp_bricks" },
-		{ gp_diag, "gp_diag" },
-		{ gp_diag_cross, "gp_diag_cross" },
-		{ gp_foreground_solid, "gp_foreground_solid" },
-		{ gp_horiz, "gp_horiz" },
-		{ gp_large_circles, "gp_large_circles" },
-		{ gp_rev_diag, "gp_rev_diag" },
-		{ gp_semi_circle, "gp_semi_circle" },
-		{ gp_small_circle, "gp_small_circle" },
-		{ gp_solid, "gp_solid" },
-		{ gp_thatch, "gp_thatch" },
-		{ gp_thick_diag_cross, "gp_thick_diag_cross" },
-		{ gp_thin_diag, "gp_thin_diag" },
-		{ gp_thin_diag_cross, "gp_thin_diag_cross" },
-		{ gp_thin_horiz, "gp_thin_horiz" },
-		{ gp_thin_horiz_cross, "gp_thin_horiz_cross" },
-		{ gp_thin_rev_diag, "gp_thin_rev_diag" },
-		{ gp_thin_vert, "gp_thin_vert" },
-		{ gp_vert, "gp_vert" },
-		{ line_pattern_dash_dot, "line_pattern_dash_dot" },
-		{ line_pattern_dash_dot_dot, "line_pattern_dash_dot_dot" },
-		{ line_pattern_dashed, "line_pattern_dashed" },
-		{ line_pattern_dotted, "line_pattern_dotted" },
-		{ line_pattern_double, "line_pattern_double" },
-		{ line_pattern_hair, "line_pattern_hair" },
-		{ line_pattern_medium, "line_pattern_medium" },
-		{ line_pattern_medium_dash, "line_pattern_medium_dash" },
-		{ line_pattern_medium_dash_dot, "line_pattern_medium_dash_dot" },
-		{ line_pattern_medium_dash_dot_dot, "line_pattern_medium_dash_dot_dot" },
-		{ line_pattern_slant, "line_pattern_slant" },
-		{ line_pattern_thick, "line_pattern_thick" },
-		{ line_pattern_thin, "line_pattern_thin" },
-		/* Borders */
-		{ bottom_border, "bottom_border" },
-		{ diag_border, "diag_border" },
-		{ inside_border, "inside_border" },
-		{ inside_horiz_border, "inside_horiz_border" },
-		{ inside_vert_border, "inside_vert_border" },
-		{ left_border, "left_border" },
-		{ no_border, "no_border" },
-		{ outline_border, "outline_border" },
-		{ rev_diag_border, "rev_diag_border" },
-		{ right_border, "right_border" },
-		{ top_border, "top_border" },
-		/* Stuff */
-		{ unknown_image, "unknown_image" },
-		{ gnumeric_splash, "gnumeric_splash" }
-	};
-	unsigned int ui;
-
-	g_return_if_fail (app != NULL);
-
-	for (ui = 0; ui < G_N_ELEMENTS (entry); ui++) {
-		GdkPixbuf *pixbuf = gdk_pixbuf_new_from_inline
-			(-1, entry[ui].scalable_data, FALSE, NULL);
-		gtk_icon_theme_add_builtin_icon (entry[ui].name,
-			gdk_pixbuf_get_width (pixbuf), pixbuf);
-		g_object_unref (pixbuf);
-	}
-}
-
-static void
-gnumeric_application_setup_icons (void)
-{
-	static struct {
-		guchar const   *scalable_data;
-		guchar const   *sized_data;
-		gchar const    *stock_id;
-	} const entry [] = {
-		{ gnm_column_add_24,			gnm_column_add_16,		"Gnumeric_ColumnAdd" },
-		{ gnm_column_delete_24,			gnm_column_delete_16,		"Gnumeric_ColumnDelete" },
-		{ gnm_column_size_24,			gnm_column_size_16,		"Gnumeric_ColumnSize" },
-		{ gnm_column_hide_24,			gnm_column_hide_16,		"Gnumeric_ColumnHide" },
-		{ gnm_column_unhide_24,			gnm_column_unhide_16,		"Gnumeric_ColumnUnhide" },
-		{ gnm_row_add_24,			gnm_row_add_16,			"Gnumeric_RowAdd" },
-		{ gnm_row_delete_24,			gnm_row_delete_16,		"Gnumeric_RowDelete" },
-		{ gnm_row_size_24,			gnm_row_size_16,		"Gnumeric_RowSize" },
-		{ gnm_row_hide_24,			gnm_row_hide_16,		"Gnumeric_RowHide" },
-		{ gnm_row_unhide_24,			gnm_row_unhide_16,		"Gnumeric_RowUnhide" },
-
-		{ gnm_group_24,				gnm_group_16,			"Gnumeric_Group" },
-		{ gnm_ungroup_24,			gnm_ungroup_16,			"Gnumeric_Ungroup" },
-		{ gnm_show_detail_24,			gnm_show_detail_16,		"Gnumeric_ShowDetail" },
-		{ gnm_hide_detail_24,			gnm_hide_detail_16,		"Gnumeric_HideDetail" },
-
-		{ gnm_graph_guru_24,			gnm_graph_guru_16,		"Gnumeric_GraphGuru" },
-		{ gnm_insert_component_24,		gnm_insert_component_16,	"Gnumeric_InsertComponent" },
-		{ gnm_insert_shaped_component_24,	gnm_insert_shaped_component_16,	"Gnumeric_InsertShapedComponent" },
-
-		{ gnm_center_across_selection_24,	gnm_center_across_selection_16,	"Gnumeric_CenterAcrossSelection" },
-		{ gnm_merge_cells_24,			gnm_merge_cells_16,		"Gnumeric_MergeCells" },
-		{ gnm_split_cells_24,			gnm_split_cells_16,		"Gnumeric_SplitCells" },
-
-		{ gnm_halign_fill_24,			NULL,				"Gnumeric_HAlignFill" },
-		{ gnm_halign_general_24,		NULL,				"Gnumeric_HAlignGeneral" },
-
-		{ NULL,					gnm_comment_add_16,		"Gnumeric_CommentAdd" },
-		{ NULL,					gnm_comment_delete_16,		"Gnumeric_CommentDelete" },
-		{ NULL,					gnm_comment_edit_16,		"Gnumeric_CommentEdit" },
-
-		{ gnm_add_decimals,			NULL,				"Gnumeric_FormatAddPrecision" },
-		{ gnm_remove_decimals,			NULL,				"Gnumeric_FormatRemovePrecision" },
-		{ gnm_money,				NULL,				"Gnumeric_FormatAsAccounting" },
-		{ gnm_percent,				NULL,				"Gnumeric_FormatAsPercentage" },
-		{ gnm_thousand,				NULL,				"Gnumeric_FormatThousandSeparator" },
-
-		{ gnm_auto,				NULL,				"Gnumeric_AutoSum" },
-		{ gnm_equal,				NULL,				"Gnumeric_Equal" },
-		{ gnm_formula_guru_24,			gnm_formula_guru_16,		"Gnumeric_FormulaGuru" },
-		{ gnm_insert_image_24,			gnm_insert_image_16,		"Gnumeric_InsertImage" },
-		{ gnm_bucket,				NULL,				"Gnumeric_Bucket" },
-		{ gnm_font,				NULL,				"Gnumeric_Font" },
-		{ gnm_expr_entry,			NULL,				"Gnumeric_ExprEntry" },
-
-		{ gnm_object_arrow_24,			NULL,				"Gnumeric_ObjectArrow" },
-		{ gnm_object_ellipse_24,		NULL,				"Gnumeric_ObjectEllipse" },
-		{ gnm_object_line_24,			NULL,				"Gnumeric_ObjectLine" },
-		{ gnm_object_rectangle_24,		NULL,				"Gnumeric_ObjectRectangle" },
-
-		{ gnm_object_frame_24,			NULL,				"Gnumeric_ObjectFrame" },
-		{ gnm_object_label_24,			NULL,				"Gnumeric_ObjectLabel" },
-		{ gnm_object_button_24,			NULL,				"Gnumeric_ObjectButton" },
-		{ gnm_object_checkbox_24,		NULL,				"Gnumeric_ObjectCheckbox" },
-		{ gnm_object_radiobutton_24,		NULL,				"Gnumeric_ObjectRadioButton" },
-		{ gnm_object_scrollbar_24,		NULL,				"Gnumeric_ObjectScrollbar" },
-		{ gnm_object_spinbutton_24,		NULL,				"Gnumeric_ObjectSpinButton" },
-		{ gnm_object_slider_24,			NULL,				"Gnumeric_ObjectSlider" },
-		{ gnm_object_combo_24,			NULL,				"Gnumeric_ObjectCombo" },
-		{ gnm_object_list_24,			NULL,				"Gnumeric_ObjectList" },
-
-		{ gnm_pivottable_24,	                gnm_pivottable_16,		"Gnumeric_PivotTable" },
-		{ gnm_protection_yes,	                NULL,				"Gnumeric_Protection_Yes" },
-		{ gnm_protection_no,       		NULL,				"Gnumeric_Protection_No" },
-		{ gnm_protection_yes_48,	        NULL,				"Gnumeric_Protection_Yes_Dialog" },
-		{ gnm_visible,	                        NULL,				"Gnumeric_Visible" },
-
-		{ gnm_link_add_24,			gnm_link_add_16,		"Gnumeric_Link_Add" },
-		{ NULL,					gnm_link_delete_16,		"Gnumeric_Link_Delete" },
-		{ NULL,					gnm_link_edit_16,		"Gnumeric_Link_Edit" },
-		{ gnm_link_external_24,			gnm_link_external_16,		"Gnumeric_Link_External" },
-		{ gnm_link_internal_24,			gnm_link_internal_16,		"Gnumeric_Link_Internal" },
-		{ gnm_link_email_24,			gnm_link_email_16,		"Gnumeric_Link_EMail" },
-		{ gnm_link_url_24,			gnm_link_url_16,		"Gnumeric_Link_URL" },
-
-		{ gnm_autofilter_24,			gnm_autofilter_16,		"Gnumeric_AutoFilter" },
-		{ gnm_autofilter_delete_24,		gnm_autofilter_delete_16,	"Gnumeric_AutoFilterDelete" },
-
-		{ gnm_border_left,			NULL,				"Gnumeric_BorderLeft" },
-		{ gnm_border_none,			NULL,				"Gnumeric_BorderNone" },
-		{ gnm_border_right,			NULL,				"Gnumeric_BorderRight" },
-
-		{ gnm_border_all,			NULL,				"Gnumeric_BorderAll" },
-		{ gnm_border_outside,			NULL,				"Gnumeric_BorderOutside" },
-		{ gnm_border_thick_outside,		NULL,				"Gnumeric_BorderThickOutside" },
-
-		{ gnm_border_bottom,			NULL,				"Gnumeric_BorderBottom" },
-		{ gnm_border_double_bottom,		NULL,				"Gnumeric_BorderDoubleBottom" },
-		{ gnm_border_thick_bottom,		NULL,				"Gnumeric_BorderThickBottom" },
-
-		{ gnm_border_top_n_bottom,		NULL,				"Gnumeric_BorderTop_n_Bottom" },
-		{ gnm_border_top_n_double_bottom,	NULL,				"Gnumeric_BorderTop_n_DoubleBottom" },
-		{ gnm_border_top_n_thick_bottom,	NULL,				"Gnumeric_BorderTop_n_ThickBottom" }
-	};
-	static gboolean done = FALSE;
-
-	if (!done) {
-		unsigned int ui = 0;
-		GtkIconFactory *factory = gtk_icon_factory_new ();
-		for (ui = 0; ui < G_N_ELEMENTS (entry) ; ui++)
-			add_icon (factory,
-				  entry[ui].scalable_data,
-				  entry[ui].sized_data,
-				  entry[ui].stock_id);
-		gtk_icon_factory_add_default (factory);
-		g_object_unref (G_OBJECT (factory));
-		done = TRUE;
-	}
-}
-
 static void
 gnumeric_application_finalize (GObject *obj)
 {
 	GnmApp *application = GNM_APP (obj);
 
-	g_slist_foreach (application->history_list, (GFunc)g_free, NULL);
-	g_slist_free (application->history_list);
-	application->history_list = NULL;
-
 	g_free (application->clipboard_cut_range);
 	application->clipboard_cut_range = NULL;
 
@@ -775,10 +655,12 @@ static void
 gnumeric_application_get_property (GObject *obj, guint param_id,
 				   GValue *value, GParamSpec *pspec)
 {
+#if 0
 	GnmApp *application = GNM_APP (obj);
+#endif
 	switch (param_id) {
 	case APPLICATION_PROP_FILE_HISTORY_LIST:
-		g_value_set_pointer (value, application->history_list);
+		g_value_set_pointer (value, gnm_app_history_get_list (FALSE));
 		break;
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 break;
@@ -794,9 +676,9 @@ gnm_app_class_init (GObjectClass *gobject_klass)
 	gobject_klass->finalize = gnumeric_application_finalize;
 	gobject_klass->get_property = gnumeric_application_get_property;
 	g_object_class_install_property (gobject_klass, APPLICATION_PROP_FILE_HISTORY_LIST,
-		g_param_spec_pointer ("file-history-list", "File History List",
-			"A GSlist of filenames that have been read recently",
-			GSF_PARAM_STATIC | G_PARAM_READABLE));
+		g_param_spec_pointer ("file-history-list", _("File History List"),
+				      _("A GSlist of filenames that have been read recently"),
+				      GSF_PARAM_STATIC | G_PARAM_READABLE));
 
 	signals [WORKBOOK_ADDED] = g_signal_new ("workbook_added",
 		GNM_APP_TYPE,
@@ -852,9 +734,6 @@ gnm_app_init (GObject *obj)
 
 	gnm_app->workbook_list = NULL;
 
-	gnumeric_application_setup_pixbufs (gnm_app);
-	gnumeric_application_setup_icons ();
-
 	app = gnm_app;
 }
 
@@ -939,4 +818,3 @@ _gnm_app_flag_windows_changed (void)
 		windows_update_timer = g_timeout_add (100,
 			(GSourceFunc)cb_flag_windows_changed, NULL);
 }
-
