@@ -56,12 +56,14 @@ cell_calc_layout (GnmCell const *cell, GnmRenderedValue *rv, int y_direction,
 	int indent;
 	int hoffset;
 	int rect_x, rect_y;
+	gboolean was_drawn;
 
 	g_return_val_if_fail (rv != NULL, FALSE);
 
 	layout = rv->layout;
 	indent = (rv->indent_left + rv->indent_right) * PANGO_SCALE;
 
+	was_drawn = rv->drawn;
 	rv->drawn = TRUE;
 
 	if (width <= 0 || height <= 0)
@@ -69,6 +71,10 @@ cell_calc_layout (GnmCell const *cell, GnmRenderedValue *rv, int y_direction,
 
 	hoffset = rv->indent_left * PANGO_SCALE;
 
+#if 0
+	g_print ("%s: w=%d  h=%d\n", cell_name (cell), width, height);
+#endif
+
 	/* This rectangle has the whole area used by this cell
 	 * excluding the surrounding grid lines and margins */
 	rect_x = PANGO_SCALE * (1 + GNM_COL_MARGIN);
@@ -92,6 +98,13 @@ cell_calc_layout (GnmCell const *cell, GnmRenderedValue *rv, int y_direction,
 		rv->hfilled = TRUE;
 	}
 
+	/* Special handling of error dates.  */
+	if (!was_drawn && rv->numeric_overflow) {
+		pango_layout_set_text (layout, hashes, -1);
+		rv->variable_width = TRUE;
+		rv->hfilled = TRUE;
+	}
+
 	if (rv->rotation && !rv->noborders) {
 		GnmRenderedRotatedValue const *rrv = (GnmRenderedRotatedValue *)rv;
 		double sin_a = rrv->rotmat.xy;
@@ -257,13 +270,19 @@ cell_draw (GnmCell const *cell, GdkGC *gc, GdkDrawable *drawable,
 	GOColor fore_color;
 	gint x;
 	gint y;
-	GnmRenderedValue *rv = cell->rendered_value;
+	GnmRenderedValue *rv;
 
 	/* Get the sizes exclusive of margins and grids */
 	/* Note: +1 because size_pixels includes leading gridline.  */
 	height -= GNM_ROW_MARGIN + GNM_ROW_MARGIN + 1;
 	width  -= GNM_COL_MARGIN + GNM_COL_MARGIN + 1;
 
+	/* render as necessary */
+	if (cell->rendered_value == NULL)
+		gnm_cell_render_value ((GnmCell *)cell, TRUE);
+
+	rv = cell->rendered_value;
+
 	if (cell_calc_layout (cell, rv, +1,
 			      width * PANGO_SCALE,
 			      height * PANGO_SCALE,
