@@ -574,6 +574,34 @@ clipboard_copy_range (Sheet *sheet, GnmRange const *r)
 	return cr;
 }
 
+static void
+cb_clipboard_copy_range_undo (GnmCellRegion *cr, GnmSheetRange *sr,
+			      GOCmdContext *cc)
+{
+	GnmPasteTarget pt;
+
+	clipboard_paste_region
+		(cr,
+		 paste_target_init (&pt,
+				    sr->sheet,
+				    &sr->range,
+				    PASTE_CONTENTS | PASTE_FORMATS),
+		 cc);
+}
+
+
+GOUndo *
+clipboard_copy_range_undo (Sheet *sheet, GnmRange const *r)
+{
+	GnmCellRegion *cr = clipboard_copy_range (sheet, r);
+	g_return_val_if_fail (cr != NULL, NULL);
+	return go_undo_binary_new (cr, gnm_sheet_range_new (sheet, r),
+				   (GOUndoBinaryFunc)cb_clipboard_copy_range_undo,
+				   (GFreeFunc)cellregion_unref,
+				   (GFreeFunc)g_free);
+}
+
+
 /**
  * clipboard_copy_obj:
  * @sheet : #Sheet
