@@ -440,7 +440,7 @@ init_button_image (GladeXML *gui, char const *name)
 
 static void
 cb_number_format_changed (G_GNUC_UNUSED GtkWidget *widget,
-			  const char *fmt,
+			  const char *lfmt,
 			  FormatState *state)
 {
 	gboolean changed = FALSE;
@@ -449,8 +449,12 @@ cb_number_format_changed (G_GNUC_UNUSED GtkWidget *widget,
 	if (!state->enable_edit)
 		return;
 
-	if (fmt) {
-	  	gnm_style_set_format_text (state->result, fmt);
+	if (lfmt) {
+		char *fmt = go_format_str_delocalize (lfmt);
+		GOFormat *format = go_format_new_from_XL (fmt);
+	  	gnm_style_set_format (state->result, format);
+		go_format_unref (format);
+		g_free (fmt);
 		changed =  TRUE;
 	}
 
@@ -2337,6 +2341,28 @@ fmt_dialog_impl (FormatState *state, FormatDialogPosition_t pageno)
 	gtk_widget_show (GTK_WIDGET (state->dialog));
 }
 
+static GnmValue *
+cb_check_cell_format (GnmCellIter const *iter, gpointer user)
+{
+	FormatState *state = user;
+	GnmValue const *value = iter->cell->value;
+	GOFormat const *common = gnm_style_get_format (state->style);
+
+	if (!value || !VALUE_FMT (value))
+		return NULL;
+
+	if (go_format_eq (common, VALUE_FMT (value)))
+		return NULL;
+
+	if (go_format_is_general (common)) {
+		gnm_style_set_format (state->style, VALUE_FMT (value));
+		return NULL;
+	} else {
+		state->conflicts |= MSTYLE_FORMAT;
+		return VALUE_TERMINATE;
+	}
+}
+
 static gboolean
 fmt_dialog_selection_type (SheetView *sv,
 			   GnmRange const *range,
@@ -2360,6 +2386,15 @@ fmt_dialog_selection_type (SheetView *sv,
 	state->conflicts = sheet_style_find_conflicts (state->sheet, range,
 		&(state->style), state->borders);
 
+	if ((state->conflicts & MSTYLE_FORMAT) == 0 &&
+	    go_format_is_general (gnm_style_get_format (state->style))) {
+		sheet_foreach_cell_in_range (state->sheet, CELL_ITER_IGNORE_BLANK,
+					     range->start.col, range->start.row,
+					     range->end.col, range->end.row,
+					     cb_check_cell_format,
+					     state);
+	}
+
 	return TRUE;
 }
 
