@@ -22,7 +22,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gnumeric.h>
 #include "dialogs.h"
 #include "help.h"
@@ -86,23 +86,13 @@ dialog_col_width_button_sensitivity (ColWidthState *state)
 	gtk_widget_set_sensitive (state->apply_button, changed_info);
 }
 
-static gboolean
-dialog_col_width_destroy (GtkObject *w, ColWidthState *state)
+static void
+cb_dialog_col_width_destroy (ColWidthState *state)
 {
-	g_return_val_if_fail (w != NULL, FALSE);
-	g_return_val_if_fail (state != NULL, FALSE);
-
 	wbcg_edit_detach_guru (state->wbcg);
-
-	if (state->gui != NULL) {
+	if (state->gui != NULL)
 		g_object_unref (G_OBJECT (state->gui));
-		state->gui = NULL;
-	}
-
-	state->dialog = NULL;
 	g_free (state);
-
-	return FALSE;
 }
 
 static void
@@ -271,8 +261,7 @@ dialog_col_width (WorkbookControlGUI *wbcg, gboolean use_default)
 
 	state->spin  = GTK_SPIN_BUTTON (glade_xml_get_widget (state->gui, "spin"));
 	gtk_spin_button_get_adjustment (state->spin)->lower =
-		state->sheet->cols.default_style.margin_a +
-		state->sheet->cols.default_style.margin_b;
+		GNM_COL_MARGIN + GNM_COL_MARGIN;
 	g_signal_connect (G_OBJECT (state->spin),
 		"value-changed",
 		G_CALLBACK (cb_dialog_col_width_value_changed), state);
@@ -299,11 +288,8 @@ dialog_col_width (WorkbookControlGUI *wbcg, gboolean use_default)
 	gnumeric_init_help_button (
 		glade_xml_get_widget (state->gui, "help_button"),
 		GNUMERIC_HELP_LINK_COL_WIDTH);
-
-	g_signal_connect (G_OBJECT (state->dialog),
-		"destroy",
-		G_CALLBACK (dialog_col_width_destroy), state);
-
+	g_object_set_data_full (G_OBJECT (state->dialog),
+		"state", state, (GDestroyNotify) cb_dialog_col_width_destroy);
 
 	dialog_col_width_set_mode (use_default, state);
 	dialog_col_width_load_value (state);
