@@ -159,7 +159,7 @@ gnm_named_expr_collection_relink (GnmNamedExprCollection *names)
 	}
 }
 
-static GnmNamedExpr *
+GnmNamedExpr *
 gnm_named_expr_collection_lookup (GnmNamedExprCollection const *scope,
 				  char const *name)
 {
@@ -592,7 +592,7 @@ expr_name_remove (GnmNamedExpr *nexpr)
  */
 char *
 expr_name_as_string (GnmNamedExpr const *nexpr, GnmParsePos const *pp,
-		     GnmExprConventions const *fmt)
+		     GnmConventions const *fmt)
 {
 	if (pp == NULL)
 		pp = &nexpr->pos;
