@@ -1,10 +1,9 @@
-/* vim: set sw=8: */
-
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * parse-util.c: Various utility routines to parse or produce
  *     string representations of common reference types.
  *
- * Copyright (C) 2000 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2000-2007 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -36,16 +35,17 @@
 #include "gnm-format.h"
 #include "expr-name.h"
 #include "str.h"
-/* For def_expr_name_handler: */
+/* For std_expr_name_handler: */
 #include "expr-impl.h"
 #include "gutils.h"
+#include <goffice/app/go-doc.h>
+#include <goffice/utils/go-glib-extras.h>
+#include <goffice/utils/go-file.h>
 
 #include <errno.h>
 #include <stdlib.h>
-#include <stdio.h>
 #include <glib.h>
 #include <string.h>
-#include <goffice/utils/go-glib-extras.h>
 
 static void
 col_name_internal (GString *target, int col)
@@ -235,10 +235,25 @@ r1c1_add_index (GString *target, char type, int num, unsigned char relative)
 		g_string_append_printf (target, "%c%d", type, num + 1);
 }
 
+static char *
+wb_rel_uri (Workbook *wb, Workbook *ref_wb)
+{
+	char const *uri = go_doc_get_uri ((GODoc *)wb);
+	char const *ref_uri = go_doc_get_uri ((GODoc *)ref_wb);
+	char *rel_uri = go_url_make_relative (uri, ref_uri);
+
+	if (rel_uri == NULL || rel_uri[0] == '/') {
+		g_free (rel_uri);
+		return g_strdup (uri);
+	}
+
+	return rel_uri;
+}
+
 /**
  * cellref_as_string :
+ * @out : #GnmConventionsOut
  * @ref :
- * @pp  :
  * @no_sheetname :
  *
  * Returns a string that the caller needs to free containing the A1 format
@@ -246,35 +261,38 @@ r1c1_add_index (GString *target, char type, int num, unsigned char relative)
  * suppress the addition of the sheetname for non-local references.
  **/
 void
-cellref_as_string (GString *target, GnmExprConventions const *conv,
+cellref_as_string (GnmConventionsOut *out,
 		   GnmCellRef const *cell_ref,
-		   GnmParsePos const *pp, gboolean no_sheetname)
+		   gboolean no_sheetname)
 {
 	int col, row;
+	GString *target = out->accum;
 	Sheet const *sheet = cell_ref->sheet;
 
 	/* If it is a non-local reference, add the path to the external sheet */
 	if (sheet != NULL && !no_sheetname) {
-		if (pp->wb == NULL && pp->sheet == NULL)
+		if (out->pp->wb == NULL && out->pp->sheet == NULL)
 			/* For the expression leak printer.  */
 			g_string_append (target, "'?'");
-		else if (pp->wb == NULL || sheet->workbook == pp->wb)
+		else if (NULL == out->pp->wb || sheet->workbook == out->pp->wb)
 			g_string_append (target, sheet->name_quoted);
 		else {
+			char *rel_uri = wb_rel_uri (sheet->workbook, out->pp->wb);
 			g_string_append_c (target, '[');
-			g_string_append (target, workbook_get_uri (sheet->workbook));
+			g_string_append (target, rel_uri);
 			g_string_append_c (target, ']');
 			g_string_append (target, sheet->name_quoted);
+			g_free (rel_uri);
 		}
-		g_string_append (target, conv->output_sheet_name_sep);
+		g_string_append_unichar (target, out->convs->sheet_name_sep);
 	}
 
-	if (conv->r1c1_addresses) { /* R1C1 handler */
+	if (out->convs->r1c1_addresses) { /* R1C1 handler */
 		r1c1_add_index (target, 'R', cell_ref->row, cell_ref->row_relative);
 		r1c1_add_index (target, 'C', cell_ref->col, cell_ref->col_relative);
 	} else {
 		if (cell_ref->col_relative)
-			col = pp->eval.col + cell_ref->col;
+			col = out->pp->eval.col + cell_ref->col;
 		else {
 			g_string_append_c (target, '$');
 			col = cell_ref->col;
@@ -287,7 +305,7 @@ cellref_as_string (GString *target, GnmExprConventions const *conv,
 		col_name_internal (target, col);
 
 		if (cell_ref->row_relative)
-			row = pp->eval.row + cell_ref->row;
+			row = out->pp->eval.row + cell_ref->row;
 		else {
 			g_string_append_c (target, '$');
 			row = cell_ref->row;
@@ -303,29 +321,31 @@ cellref_as_string (GString *target, GnmExprConventions const *conv,
 
 /**
  * rangeref_as_string :
- * @ref :
- * @pp :
+ * @out : #GnmConventionsOut
+ * @ref : #GnmRangeRef
  *
  **/
 void
-rangeref_as_string (GString *target, GnmExprConventions const *conv,
-		    GnmRangeRef const *ref, GnmParsePos const *pp)
+rangeref_as_string (GnmConventionsOut *out, GnmRangeRef const *ref)
 {
 	GnmRange r;
+	GString *target = out->accum;
 
-	r.start.col = cellref_abs_col (&ref->a, pp);
-	r.end.col   = cellref_abs_col (&ref->b, pp);
-	r.start.row = cellref_abs_row (&ref->a, pp);
-	r.end.row   = cellref_abs_row (&ref->b, pp);
+	r.start.col = cellref_abs_col (&ref->a, out->pp);
+	r.end.col   = cellref_abs_col (&ref->b, out->pp);
+	r.start.row = cellref_abs_row (&ref->a, out->pp);
+	r.end.row   = cellref_abs_row (&ref->b, out->pp);
 
 	if (ref->a.sheet) {
-		if (pp->wb != NULL && ref->a.sheet->workbook != pp->wb) {
+		if (NULL != out->pp->wb && ref->a.sheet->workbook != out->pp->wb) {
+			char *rel_uri = wb_rel_uri (ref->a.sheet->workbook, out->pp->wb);
 			g_string_append_c (target, '[');
-			g_string_append (target, workbook_get_uri (ref->a.sheet->workbook));
+			g_string_append (target, rel_uri);
 			g_string_append_c (target, ']');
+			g_free (rel_uri);
 		}
-		if (pp->wb == NULL && pp->sheet == NULL)
-			/* For the expression leak printer. */
+		if (out->pp->wb == NULL && out->pp->sheet == NULL)
+			/* For the expression leak printer.  */
 			g_string_append (target, "'?'");
 		else if (ref->b.sheet == NULL || ref->a.sheet == ref->b.sheet)
 			g_string_append (target, ref->a.sheet->name_quoted);
@@ -334,10 +354,10 @@ rangeref_as_string (GString *target, GnmExprConventions const *conv,
 			g_string_append_c (target, ':');
 			g_string_append (target, ref->b.sheet->name_quoted);
 		}
-		g_string_append (target, conv->output_sheet_name_sep);
+		g_string_append_unichar (target, out->convs->sheet_name_sep);
 	}
 
-	if (conv->r1c1_addresses) { /* R1C1 handler */
+	if (out->convs->r1c1_addresses) { /* R1C1 handler */
 		/* be sure to use else if so that a1:iv65535 does not vanish */
 		if (r.start.col == 0 && r.end.col == SHEET_MAX_COLS-1) {
 			r1c1_add_index (target, 'R', ref->a.row, ref->a.row_relative);
@@ -409,30 +429,33 @@ rangeref_as_string (GString *target, GnmExprConventions const *conv,
 
 /**
  * gnm_1_0_rangeref_as_string :
- * @ref :
- * @pp :
+ * @out : #GnmConventionsOut
+ * @ref : #GnmRangeRef
  *
- * Simplified variant of rangeref_as_string that old version of gnumeric can
- * read.  It drops support for full col/row references.  We can remap them on import.
+ * Simplified variant of rangeref_as_string that old versions of gnumeric can
+ * read.  It drops support for full col/row references.  We can remap them on
+ * import.
  **/
 void
-gnm_1_0_rangeref_as_string (GString *target, GnmExprConventions const *conv,
-			    GnmRangeRef const *ref, GnmParsePos const *pp)
+gnm_1_0_rangeref_as_string (GnmConventionsOut *out, GnmRangeRef const *ref)
 {
 	GnmRange r;
+	GString *target = out->accum;
 
-	r.start.col = cellref_abs_col (&ref->a, pp);
-	r.end.col   = cellref_abs_col (&ref->b, pp);
-	r.start.row = cellref_abs_row (&ref->a, pp);
-	r.end.row   = cellref_abs_row (&ref->b, pp);
+	r.start.col = cellref_abs_col (&ref->a, out->pp);
+	r.end.col   = cellref_abs_col (&ref->b, out->pp);
+	r.start.row = cellref_abs_row (&ref->a, out->pp);
+	r.end.row   = cellref_abs_row (&ref->b, out->pp);
 
 	if (ref->a.sheet) {
-		if (pp->wb != NULL && ref->a.sheet->workbook != pp->wb) {
+		if (NULL != out->pp->wb && ref->a.sheet->workbook != out->pp->wb) {
+			char *rel_uri = wb_rel_uri (ref->a.sheet->workbook, out->pp->wb);
 			g_string_append_c (target, '[');
-			g_string_append (target, workbook_get_uri (ref->a.sheet->workbook));
+			g_string_append (target, rel_uri);
 			g_string_append_c (target, ']');
+			g_free (rel_uri);
 		}
-		if (pp->wb == NULL && pp->sheet == NULL)
+		if (out->pp->wb == NULL && out->pp->sheet == NULL)
 			/* For the expression leak printer. */
 			g_string_append (target, "'?'");
 		else if (ref->b.sheet == NULL || ref->a.sheet == ref->b.sheet)
@@ -442,7 +465,7 @@ gnm_1_0_rangeref_as_string (GString *target, GnmExprConventions const *conv,
 			g_string_append_c (target, ':');
 			g_string_append (target, ref->b.sheet->name_quoted);
 		}
-		g_string_append (target, conv->output_sheet_name_sep);
+		g_string_append_unichar (target, out->convs->sheet_name_sep);
 	}
 
 	if (!ref->a.col_relative)
@@ -535,13 +558,17 @@ static char const *
 cellref_r1c1_get (GnmCellRef *out, char const *in, GnmCellPos const *pos)
 {
 	out->sheet = NULL;
-	if (*in != 'R')
+	if (*in != 'R' && *in != 'r')
 		return NULL;
 	if (NULL == (in = r1c1_get_index (in, &out->row, &out->row_relative, FALSE)))
 		return NULL;
-	if (*in != 'C')
+	if (*in != 'C' && *in != 'c')
 		return NULL;
-	return r1c1_get_index (in, &out->col, &out->col_relative, TRUE);
+	if (NULL == (in = r1c1_get_index (in, &out->col, &out->col_relative, TRUE)))
+		return NULL;
+	if (g_ascii_isalpha (*in))
+		return NULL;
+	return in;
 }
 
 /**
@@ -572,18 +599,30 @@ cellref_parse (GnmCellRef *out, char const *in, GnmCellPos const *pos)
 
 /****************************************************************************/
 
+static char const *
+cell_coord_name2 (int col, int row, gboolean r1c1)
+{
+	static GString *buffer = NULL;
+	if (buffer)
+		g_string_truncate (buffer, 0);
+	else
+		buffer = g_string_new (NULL);
+
+	if (r1c1) {
+		r1c1_add_index (buffer, 'R', row, FALSE);
+		r1c1_add_index (buffer, 'C', col, FALSE);
+	} else {
+		col_name_internal (buffer, col);
+		row_name_internal (buffer, row);
+	}
+
+	return buffer->str;
+}
+
 char const *
 cell_coord_name (int col, int row)
 {
-	static GString *buffer = NULL;
-	if (!buffer)
-		buffer = g_string_new (NULL);
-	g_string_truncate (buffer, 0);
-
-	col_name_internal (buffer, col);
-	row_name_internal (buffer, row);
-
-	return buffer->str;
+	return cell_coord_name2 (col, row, FALSE);
 }
 
 char const *
@@ -594,12 +633,24 @@ cellpos_as_string (GnmCellPos const *pos)
 	return cell_coord_name (pos->col, pos->row);
 }
 
+char const *
+parsepos_as_string (GnmParsePos const *pp)
+{
+	g_return_val_if_fail (pp != NULL, "ERROR");
+
+	return cell_coord_name2 (pp->eval.col,
+				 pp->eval.row,
+				 pp->sheet && pp->sheet->r1c1_addresses);
+}
+
 char const *
 cell_name (GnmCell const *cell)
 {
 	g_return_val_if_fail (cell != NULL, "ERROR");
 
-	return cell_coord_name (cell->pos.col, cell->pos.row);
+	return cell_coord_name2 (cell->pos.col,
+				 cell->pos.row,
+				 cell->base.sheet->r1c1_addresses);
 }
 
 /**
@@ -667,7 +718,6 @@ gnm_expr_char_start_p (char const * c)
 		 * numbers to be treated as formulae.  FIXME: this really
 		 * just checks for C-syntax numbers.
 		 */
-		errno = 0;
 		(void) gnm_strto (c, &end);
 		if (errno || *end != 0 || end == c)
 			return (c0 == '+') ? c + 1 : c;
@@ -682,8 +732,8 @@ gnm_expr_char_start_p (char const * c)
  *
  * @pos : If the string looks like an expression parse it at this location.
  * @text: The text to be parsed.
- * @val : Returns a GnmValue * if the text was a value, otherwise NULL.
- * @expr: Returns an GnmExpr * if the text was an expression, otherwise NULL.
+ * @val : Returns a GnmValue* if the text was a value, otherwise NULL.
+ * @texpr: Returns a GnmExprTop* if the text was an expression, otherwise NULL.
  * @cur_fmt : Optional, current number format.
  * @date_conv : Optional, date parse conventions
  *
@@ -692,13 +742,13 @@ gnm_expr_char_start_p (char const * c)
  */
 void
 parse_text_value_or_expr (GnmParsePos const *pos, char const *text,
-			  GnmValue **val, GnmExpr const **expr,
+			  GnmValue **val, GnmExprTop const **texpr,
 			  GOFormat *cur_fmt,
 			  GODateConventions const *date_conv)
 {
 	char const *expr_start;
 
-	*expr = NULL;
+	*texpr = NULL;
 
 	/* Does it match any formats?  */
 	*val = format_match (text, cur_fmt, date_conv);
@@ -708,11 +758,9 @@ parse_text_value_or_expr (GnmParsePos const *pos, char const *text,
 	/* If it does not match known formats, see if it is an expression */
 	expr_start = gnm_expr_char_start_p (text);
 	if (NULL != expr_start && *expr_start) {
-		*expr = gnm_expr_parse_str (expr_start, pos,
-					    GNM_EXPR_PARSE_DEFAULT,
-					    gnm_expr_conventions_default,
-					    NULL);
-		if (*expr != NULL)
+		*texpr = gnm_expr_parse_str (expr_start, pos,
+			GNM_EXPR_PARSE_DEFAULT, NULL, NULL);
+		if (*texpr != NULL)
 			return;
 	}
 
@@ -785,7 +833,7 @@ unquote (char *dst, char const *src, int n)
  *           Otherwise returns @start and does not modify @wb.
  * **/
 static char const *
-wbref_parse (char const *start, Workbook **wb)
+wbref_parse (char const *start, Workbook **wb, Workbook *ref_wb)
 {
 	/* Is this an external reference ? */
 	if (*start == '[') {
@@ -811,7 +859,9 @@ wbref_parse (char const *start, Workbook **wb)
 		} else
 			unquote (name, start+2, end-start-2);
 
-		tmp_wb = gnm_app_workbook_get_by_name (name);
+		tmp_wb = gnm_app_workbook_get_by_name
+			(name,
+			 ref_wb ? go_doc_get_uri ((GODoc *)ref_wb) : NULL);
 		if (tmp_wb == NULL)
 			return NULL;
 		*wb = tmp_wb;
@@ -849,10 +899,47 @@ sheetref_parse (char const *start, Sheet **sheet, Workbook const *wb,
 			return start;
 		}
 	} else {
-		for (end = start;
-		     g_unichar_isalnum (g_utf8_get_char (end));
-		     end = g_utf8_next_char (end))
-			; /* Nothing */
+		gboolean only_digits = TRUE;
+		end = start;
+
+		/*
+		 * Valid: Normal!a1
+		 * Valid: x.y!a1
+		 * Invalid: .y!a1
+		 *
+		 * Some names starting with digits are actually valid, but
+		 * unparse quoted. Things are quite tricky: most sheet names
+		 * starting with a digit are ok, but not those starting with
+		 * "[0-9]*\." or "[0-9]+[eE]".
+		 * 
+		 * Valid: 42!a1
+		 * Valid: 4x!a1
+		 * Invalid: 1.!a1
+		 * Invalid: 1e!a1
+		 */
+
+		while (1) {
+			gunichar uc = g_utf8_get_char (end);
+			if (g_unichar_isalpha (uc) || uc == '_') {
+				if (only_digits && end != start &&
+				    (uc == 'e' || uc == 'E')) {
+					end = start;
+					break;
+				}
+				only_digits = FALSE;
+				end = g_utf8_next_char (end);
+			} else if (g_unichar_isdigit (uc)) {
+				end = g_utf8_next_char (end);
+			} else if (uc == '.') {
+				/* Valid, except after only digits.  */
+				if (only_digits) {
+					end = start;
+					break;
+				}
+				end++;
+			} else 
+				break;
+		}
 
 		if (*end != '!' && (!allow_3d || *end != ':'))
 			return start;
@@ -868,8 +955,6 @@ sheetref_parse (char const *start, Sheet **sheet, Workbook const *wb,
 	return end;
 }
 
-/* An odd little routine that is called when we start parsing looking for an A1
- * style ref and get into a situation that may be an R1C1 */
 static char const *
 r1c1_rangeref_parse (GnmRangeRef *res, char const *ptr, GnmParsePos const *pp)
 {
@@ -878,11 +963,14 @@ r1c1_rangeref_parse (GnmRangeRef *res, char const *ptr, GnmParsePos const *pp)
 	if (*ptr == 'R' || *ptr == 'r') {
 		if (NULL == (ptr = r1c1_get_index (ptr, &res->a.row, &res->a.row_relative, FALSE)))
 			return NULL;
-		if (*ptr != 'C' && *ptr != 'c') { /* full row R# */
-			res->a.col_relative = res->b.col_relative = FALSE;
-			res->a.col = 0; res->b.col = SHEET_MAX_COLS-1;
-			res->b.row_relative = res->a.row_relative;
-			res->b.row = res->a.row;
+		if (*ptr != 'C' && *ptr != 'c') {
+			if (g_ascii_isalpha (*ptr))
+				return NULL;
+			/* full row R# */
+			res->a.col_relative = FALSE;
+			res->a.col = 0;
+			res->b = res->a;
+			res->b.col = SHEET_MAX_COLS - 1;
 			if (ptr[0] != ':' || (ptr[1] != 'R' && ptr[1] != 'r'))
 				return ptr;
 			if (NULL == (tmp = r1c1_get_index (ptr+1, &res->b.row, &res->b.row_relative, FALSE)))
@@ -898,13 +986,16 @@ r1c1_rangeref_parse (GnmRangeRef *res, char const *ptr, GnmParsePos const *pp)
 		    NULL == (tmp = r1c1_get_index (tmp, &res->b.col, &res->b.col_relative, FALSE)))
 			return ptr;
 		return tmp;
-	} else if (*ptr == 'C' || *ptr == 'c') { /* full col C[#] */
+	} else if (*ptr == 'C' || *ptr == 'c') {
 		if (NULL == (ptr = r1c1_get_index (ptr, &res->a.col, &res->a.col_relative, TRUE)))
 			return NULL;
-		res->a.row_relative = res->b.row_relative = FALSE;
-		res->a.row = 0; res->b.row = SHEET_MAX_ROWS-1;
-		res->b.col_relative = res->a.col_relative;
-		res->b.col = res->a.col;
+		if (g_ascii_isalpha (*ptr))
+			return NULL;
+		 /* full col C[#] */
+		res->a.row_relative = FALSE;
+		res->a.row = 0; 
+		res->b = res->a;
+		res->b.row = SHEET_MAX_ROWS - 1;
 		if (ptr[0] != ':' || (ptr[1] != 'C' && ptr[1] != 'c'))
 			return ptr;
 		if (NULL == (tmp = r1c1_get_index (ptr, &res->b.col, &res->b.col_relative, TRUE)))
@@ -925,16 +1016,18 @@ r1c1_rangeref_parse (GnmRangeRef *res, char const *ptr, GnmParsePos const *pp)
  **/
 char const *
 rangeref_parse (GnmRangeRef *res, char const *start, GnmParsePos const *pp,
-		GnmExprConventions const *convs)
+		GnmConventions const *convs)
 {
 	char const *ptr = start, *start_sheet, *tmp1, *tmp2;
 	Workbook *wb;
+	Workbook *ref_wb;
 
 	g_return_val_if_fail (start != NULL, start);
 	g_return_val_if_fail (pp != NULL, start);
 
 	wb = pp->wb;
-	start_sheet = wbref_parse (start, &wb);
+	ref_wb = wb ? wb : pp->sheet->workbook;
+	start_sheet = wbref_parse (start, &wb, ref_wb);
 	if (start_sheet == NULL)
 		return start; /* TODO error unknown workbook */
 	ptr = sheetref_parse (start_sheet, &res->a.sheet, wb, TRUE);
@@ -1023,7 +1116,7 @@ rangeref_parse (GnmRangeRef *res, char const *start, GnmParsePos const *pp,
 /* Do we even need this anymore ? */
 char const *
 gnm_1_0_rangeref_parse (GnmRangeRef *res, char const *start, GnmParsePos const *pp,
-			G_GNUC_UNUSED GnmExprConventions const *convs)
+			G_GNUC_UNUSED GnmConventions const *convs)
 {
 	char const *ptr = start, *tmp1, *tmp2;
 	Workbook *wb;
@@ -1032,7 +1125,7 @@ gnm_1_0_rangeref_parse (GnmRangeRef *res, char const *start, GnmParsePos const *
 	g_return_val_if_fail (pp != NULL, start);
 
 	wb = pp->wb;
-	ptr = wbref_parse (start, &wb);
+	ptr = wbref_parse (start, &wb, NULL);
 	if (ptr == NULL)
 		return start; /* TODO error unknown workbook */
 
@@ -1078,33 +1171,33 @@ gnm_1_0_rangeref_parse (GnmRangeRef *res, char const *start, GnmParsePos const *
 /* ------------------------------------------------------------------------- */
 
 static void
-def_expr_name_handler (GString *target,
-		       GnmParsePos const *pp,
-		       GnmExprName const *name,
-		       GnmExprConventions const *conv)
+std_expr_name_handler (GnmConventionsOut *out, GnmExprName const *name)
 {
 	GnmNamedExpr const *thename = name->name;
+	GString *target = out->accum;
 
 	if (!thename->active) {
 		g_string_append (target,
-				 value_error_name (GNM_ERROR_REF, conv->output_translated));
+			value_error_name (GNM_ERROR_REF, out->convs->output.translated));
 		return;
 	}
 
 	if (name->optional_scope != NULL) {
-		if (name->optional_scope->workbook != pp->wb) {
+		if (name->optional_scope->workbook != out->pp->wb) {
+			char *rel_uri = wb_rel_uri (name->optional_wb_scope, out->pp->wb);
 			g_string_append_c (target, '[');
-			g_string_append (target, workbook_get_uri (name->optional_wb_scope));
+			g_string_append (target, rel_uri);
 			g_string_append_c (target, ']');
+			g_free (rel_uri);
 		} else {
 			g_string_append (target, name->optional_scope->name_quoted);
-			g_string_append (target, conv->output_sheet_name_sep);
+			g_string_append_unichar (target, out->convs->sheet_name_sep);
 		}
-	} else if (pp->sheet != NULL &&
+	} else if (out->pp->sheet != NULL &&
 		   thename->pos.sheet != NULL &&
-		   thename->pos.sheet != pp->sheet) {
+		   thename->pos.sheet != out->pp->sheet) {
 		g_string_append (target, thename->pos.sheet->name_quoted);
-		g_string_append (target, conv->output_sheet_name_sep);
+		g_string_append_unichar (target, out->convs->sheet_name_sep);
 	}
 
 	g_string_append (target, thename->name->str);
@@ -1112,25 +1205,166 @@ def_expr_name_handler (GString *target,
 
 /* ------------------------------------------------------------------------- */
 
-GnmExprConventions *
-gnm_expr_conventions_new (void)
+static GString *
+std_sheet_name_quote (GnmConventions const *convs,
+		      char const *str)
 {
-	GnmExprConventions *res = g_new0 (GnmExprConventions, 1);
+	gunichar uc = g_utf8_get_char (str);
+	GString *res = g_string_sized_new (20);
+	char const *p;
+	int nletters;
+	int ndigits;
+
+	if (g_ascii_isalpha (uc)) {
+		nletters = 1;
+		ndigits = 0;
+		p = str + 1;
+	} else if (g_unichar_isalpha (uc) || uc == '_') {
+		nletters = -1;
+		ndigits = -1;
+		p = g_utf8_next_char (str);
+	} else
+		goto quoted;
+
+	/* FIXME: What about '?' and '\\'.  I cannot enter those.  */
+
+	for (; *p; p = g_utf8_next_char (p)) {
+		uc = g_utf8_get_char (p);
+
+		if (g_ascii_isalpha (uc)) {
+			if (ndigits == 0)
+				nletters++;
+		} else if (g_ascii_isdigit (uc)) {
+			if (ndigits >= 0)
+				ndigits++;
+		} else if (uc == '.' || uc == '_' || g_unichar_isalpha (uc))
+			nletters = ndigits = -1;
+		else
+			goto quoted;
+	}
+
+	if (ndigits > 0) {
+		/*
+		 * Excel also quotes things that look like cell references.
+		 * Precisely, check for a match against
+		 *    ([A-Za-z]+)0*([1-9][0-9]*)
+		 * where $1 is a valid column name and $2 is a valid row
+		 * number.  (The 0* is an Excel bug.)
+		 */
+
+		int col, row;
+		unsigned char col_relative, row_relative;
+		if (!col_parse (str, &col, &col_relative))
+			goto unquoted;
+
+		p = str + nletters;
+		while (*p == '0')
+			p++, ndigits--;
+		if (!row_parse (p, &row, &row_relative))
+			goto unquoted;
+
+		goto quoted;
+	}
+
+ unquoted:
+	g_string_append (res, str);
+	return res;
+
+ quoted:
+	g_string_append_c (res, '\'');
+	/* This is UTF-8 safe.  */
+	for (; *str; str++) {
+		gchar c = *str;
+		if (c == '\'' || c == '\\')
+			g_string_append_c (res, '\\');
+		g_string_append_c (res, c);
+	}
+	g_string_append_c (res, '\'');
 
-	res->expr_name_handler = def_expr_name_handler;
-	res->cell_ref_handler = cellref_as_string;
-	res->range_ref_handler = rangeref_as_string;
-	res->output_sheet_name_sep = "!";
-	res->output_translated = TRUE;
 	return res;
 }
 
-void
-gnm_expr_conventions_free (GnmExprConventions *c)
+static char const *
+std_name_parser (char const *str,
+		 G_GNUC_UNUSED GnmConventions const *convs)
 {
-	if (c->function_rewriter_hash)
-		g_hash_table_destroy (c->function_rewriter_hash);
+	gunichar uc = g_utf8_get_char (str);
 
+	if (!g_unichar_isalpha (uc) && uc != '_' && uc != '\\')
+		return NULL;
+
+	do {
+		str = g_utf8_next_char (str);
+		uc = g_utf8_get_char (str);
+	} while (g_unichar_isalnum (uc) ||
+		 uc == '_' ||
+		 uc == '?' ||
+		 uc == '\\' ||
+		 uc == '.');
+
+	return str;
+}
+
+static GnmExpr const *
+std_func_map (GnmConventions const *convs, Workbook *scope,
+	      char const *name, GnmExprList *args)
+{
+	GnmFunc *f;
+
+	if (NULL == (f = gnm_func_lookup (name, scope)))
+		f = gnm_func_add_placeholder (scope, name, "", TRUE);
+	return gnm_expr_new_funcall (f, args);
+}
+
+/**
+ * gnm_conventions_new_full :
+ * @size :
+ *
+ * Construct a GnmConventions of @size.
+ *
+ * Returns a GnmConventions with default values.  Caller is responsible for
+ * freeing the result.
+ **/
+GnmConventions *
+gnm_conventions_new_full (unsigned size)
+{
+	GnmConventions *convs;
+
+	g_return_val_if_fail (size >= sizeof (GnmConventions), NULL);
+
+	convs = g_malloc0 (size);
+
+	convs->sheet_name_sep		= '!';
+	convs->intersection_char		= ' ';
+	convs->input.name		= std_name_parser;
+	convs->input.func		= std_func_map;
+
+	convs->output.translated		= TRUE;
+	convs->output.name		= std_expr_name_handler;
+	convs->output.cell_ref		= cellref_as_string;
+	convs->output.range_ref		= rangeref_as_string;
+	convs->output.quote_sheet_name	= std_sheet_name_quote;
+	return convs;
+}
+
+/**
+ * gnm_conventions_new :
+ *
+ * A convenience wrapper around gnm_conventions_new_full
+ * that constructs a GnmConventions of std size.
+ *
+ * Returns a GnmConventions with default values.  Caller is responsible for
+ * freeing the result.
+ **/
+GnmConventions *
+gnm_conventions_new (void)
+{
+	return gnm_conventions_new_full (sizeof (GnmConventions));
+}
+
+void
+gnm_conventions_free (GnmConventions *c)
+{
 	g_free (c);
 }
 
@@ -1243,49 +1477,66 @@ test_cellpos_stuff (void)
 
 #endif
 
-GnmExprConventions const *gnm_expr_conventions_default;
-GnmExprConventions const *gnm_expr_conventions_r1c1;
+GnmConventions const *gnm_conventions_default;
+GnmConventions const *gnm_conventions_xls_r1c1;
 
 void
 parse_util_init (void)
 {
-	GnmExprConventions *convs;
+	GnmConventions *convs;
 #ifdef TEST
 	test_row_stuff ();
 	test_col_stuff ();
 	test_cellpos_stuff ();
 #endif
 
-	convs = gnm_expr_conventions_new ();
-	convs->ref_parser = rangeref_parse;
-	convs->range_sep_colon		= TRUE;
-	convs->sheet_sep_exclamation	= TRUE;
-	convs->dots_in_names		= TRUE;
-	gnm_expr_conventions_default = convs;
+	convs = gnm_conventions_new ();
+	convs->input.range_ref = rangeref_parse;
+	convs->range_sep_colon		 = TRUE;
+	convs->r1c1_addresses		 = FALSE;
+	gnm_conventions_default	 = convs;
 
-	convs = gnm_expr_conventions_new ();
-	convs->ref_parser = rangeref_parse;
-	convs->range_sep_colon		= TRUE;
-	convs->sheet_sep_exclamation	= TRUE;
-	convs->dots_in_names		= TRUE;
-	convs->r1c1_addresses		= TRUE;
-	gnm_expr_conventions_r1c1 = convs;
+	convs = gnm_conventions_new ();
+	convs->input.range_ref = rangeref_parse;
+	convs->range_sep_colon		 = TRUE;
+	convs->r1c1_addresses		 = TRUE;
+	gnm_conventions_xls_r1c1	 = convs;
 }
 
 void
 parse_util_shutdown (void)
 {
-	gnm_expr_conventions_free ((GnmExprConventions *)gnm_expr_conventions_default);
-	gnm_expr_conventions_default = NULL;
-	gnm_expr_conventions_free ((GnmExprConventions *)gnm_expr_conventions_r1c1);
-	gnm_expr_conventions_r1c1 = NULL;
+	gnm_conventions_free ((GnmConventions *)gnm_conventions_default);
+	gnm_conventions_default = NULL;
+	gnm_conventions_free ((GnmConventions *)gnm_conventions_xls_r1c1);
+	gnm_conventions_xls_r1c1 = NULL;
 }
 
-GnmExpr const *
+GnmExprTop const *
 gnm_expr_parse_str_simple (char const *expr, GnmParsePos const *pp)
 {
-	return gnm_expr_parse_str (expr, pp, GNM_EXPR_PARSE_DEFAULT,
-				   gnm_expr_conventions_default, NULL);
+	return gnm_expr_parse_str (expr, pp, GNM_EXPR_PARSE_DEFAULT, NULL, NULL);
 }
 
 /* ------------------------------------------------------------------------- */
+/**
+ * gnm_expr_conv_quote:
+ * @convs : #GnmConventions
+ * @str   : string to quote
+ *
+ * Quotes @str according to the convention @convs if necessary. 
+ * or returns a literal copy of @str if no quoting was needed.
+ *
+ * Return value: caller is responsible for the resulting GString 
+ **/
+GString *
+gnm_expr_conv_quote (GnmConventions const *convs,
+		     char const *str)
+{
+	g_return_val_if_fail (convs != NULL, NULL);
+	g_return_val_if_fail (convs->output.quote_sheet_name != NULL, NULL);
+	g_return_val_if_fail (str != NULL, NULL);
+	g_return_val_if_fail (str[0] != 0, NULL);
+
+	return convs->output.quote_sheet_name (convs, str);
+}
