@@ -5,6 +5,7 @@
  * Author:
  *    Jody Goldberg 2000-2006	(jody@gnome.org)
  *    Miguel de Icaza 1999 (miguel@kernel.org)
+ *    Andreas J. Guelzow 2007   (aguelzow@pyrshep.ca)
  */
 #include <gnumeric-config.h>
 #include "gnumeric.h"
@@ -29,7 +30,6 @@
 #include "rendered-value.h"
 #include "str.h"
 #include "cell-draw.h"
-#include <libgnomeprint/gnome-print-pango.h>
 
 #include <string.h>
 #include <locale.h>
@@ -40,31 +40,6 @@
 #define MERGE_DEBUG(range, str)
 #endif
 
-/*
- * print_make_rectangle_path
- * @pc      print context
- * @left    left side x coordinate
- * @bottom  bottom side y coordinate
- * @right   right side x coordinate
- * @top     top side y coordinate
- *
- * Make a rectangular path.
- */
-void
-print_make_rectangle_path (GnomePrintContext *pc,
-			   double left, double bottom,
-			   double right, double top)
-{
-	g_return_if_fail (pc != NULL);
-
-	gnome_print_newpath   (pc);
-	gnome_print_moveto    (pc, left, bottom);
-	gnome_print_lineto    (pc, left, top);
-	gnome_print_lineto    (pc, right, top);
-	gnome_print_lineto    (pc, right, bottom);
-	gnome_print_closepath (pc);
-}
-
 /*
  * base_[xy] : Coordinates of the upper left corner of the cell.
  *             INCLUSIVE of the near grid line
@@ -75,12 +50,13 @@ print_make_rectangle_path (GnomePrintContext *pc,
  *      |      |
  *      \------/
  */
+
 static void
-print_cell (GnmCell const *cell, GnmStyle const *mstyle,
-	    GnomePrintContext *context, PangoContext *pcontext,
+print_cell_gtk (GnmCell const *cell, GnmStyle const *mstyle,
+	    cairo_t *context, PangoContext *pcontext,
 	    double x1, double y1, double width, double height, double h_center)
 {
-	RenderedValue *rv, *cell_rv = cell->rendered_value, *cell_rv100 = NULL;
+	GnmRenderedValue *rv, *cell_rv = cell->rendered_value, *cell_rv100 = NULL;
 	GOColor fore_color;
 	gint x, y;
 	Sheet *sheet = cell->base.sheet;
@@ -91,13 +67,13 @@ print_cell (GnmCell const *cell, GnmStyle const *mstyle,
 	width  -= GNM_COL_MARGIN + GNM_COL_MARGIN + 1;
 
 	/* Create a rendered value for printing */
-	if (sheet->last_zoom_factor_used != 1) {
+	if (sheet->last_zoom_factor_used != 1 || cell_rv == NULL) {
 		/*
 		 * We're zoomed and we don't want printing to reflect that.
-		 * Simply create a new RenderedValue at zoom 100% for the
+		 * Simply create a new GnmRenderedValue at zoom 100% for the
 		 * _screen_ context.
 		 */
-		cell_rv100 = rendered_value_new ((GnmCell *)cell, mstyle,
+		cell_rv100 = gnm_rendered_value_new ((GnmCell *)cell, mstyle,
 			cell_rv->variable_width,
 			pango_layout_get_context (cell_rv->layout),
 			1.0);
@@ -111,14 +87,14 @@ print_cell (GnmCell const *cell, GnmStyle const *mstyle,
 	cell_finish_layout (cell, cell_rv, width, FALSE);
 
 	/* Now pretend it was made for printing.  */
-	rv = rendered_value_recontext (cell_rv, pcontext);
+	rv = gnm_rendered_value_recontext (cell_rv, pcontext);
 
 	/* Make sure we don't get overflow in print unless we had it in
 	   display.  */
 	rv->might_overflow = rv->numeric_overflow;
 
 	if (cell_rv100)
-		rendered_value_destroy (cell_rv100);
+		gnm_rendered_value_destroy (cell_rv100);
 
 	if (cell_calc_layout (cell, rv, -1,
 			      (int)(width * PANGO_SCALE),
@@ -126,82 +102,64 @@ print_cell (GnmCell const *cell, GnmStyle const *mstyle,
 			      (int)h_center == -1 ? -1 : (int)(h_center * PANGO_SCALE),
 			      &fore_color, &x, &y)) {
 		double x0 = x1 + (1 + GNM_COL_MARGIN);
-		double y0 = y1 - (1 + GNM_ROW_MARGIN);
+		double y0 = y1 + (1 + GNM_ROW_MARGIN);
 		double px = x1 + x / (double)PANGO_SCALE;
-		double py = y1 + y / (double)PANGO_SCALE;
+		double py = y1 - y / (double)PANGO_SCALE;
 
-		/* Clip the printed rectangle */
-		gnome_print_gsave (context);
+ 		/* Clip the printed rectangle */
+ 		cairo_save (context); 
 
 		if (!rv->rotation) {
 			/* We do not clip rotated cells.  */
-			print_make_rectangle_path (context,
-						   x0 - 1, y0 - height,
-						   x0 + width, y0 + 1);
-			gnome_print_clip (context);
+			cairo_new_path (context);
+			cairo_rectangle (context, x0 - 1, y0 -1,
+					 width + 1, height + 1);
+			cairo_clip (context);
 		}
 
 		/* Set the font colour */
-		gnome_print_setrgbcolor (context,
-			UINT_RGBA_R (fore_color) / 255.,
-			UINT_RGBA_G (fore_color) / 255.,
-			UINT_RGBA_B (fore_color) / 255.);
+		cairo_set_source_rgb (context,
+			 UINT_RGBA_R (fore_color) / 255.,
+			 UINT_RGBA_G (fore_color) / 255.,
+			 UINT_RGBA_B (fore_color) / 255.);
 
 		if (rv->rotation) {
-			RenderedRotatedValue *rrv = (RenderedRotatedValue *)rv;
-			const struct RenderedRotatedValueInfo *li = rrv->lines;
-			GSList *lines;
-
-			for (lines = pango_layout_get_lines (rv->layout);
-			     lines;
-			     lines = lines->next, li++) {
-				gnome_print_gsave (context);
-				gnome_print_moveto (context,
-						    px + li->dx / (double)PANGO_SCALE,
-						    py - li->dy / (double)PANGO_SCALE);
-				gnome_print_rotate (context, rv->rotation);
-				gnome_print_pango_layout_line (context, lines->data);
-				gnome_print_grestore (context);
-			}
+			cairo_save (context);
+			cairo_translate (context, x1, y1);
+			cairo_rotate (context, - rv->rotation/180.*3.14159);
+			cairo_move_to (context, 0.,0.);
+			pango_cairo_show_layout (context, rv->layout);
+			cairo_restore (context);
 		} else {
-			gnome_print_moveto (context, px, py);
-			gnome_print_pango_layout (context, rv->layout);
+			cairo_move_to (context, px, py);
+			pango_cairo_show_layout (context, rv->layout);
 		}
-
-		gnome_print_grestore (context);
+		cairo_restore(context);
 	}
 
-	rendered_value_destroy (rv);
+	gnm_rendered_value_destroy (rv);
 }
 
-/* We do not use print_make_rectangle_path here - because we do not want a
- * new path.  */
 static void
-print_rectangle (GnomePrintContext *context,
+print_rectangle_gtk (cairo_t *context,
 		 double x, double y, double w, double h)
 {
-	/* Mirror gdk which excludes the far point */
-	w -= 1.;
-	h -= 1.;
-	gnome_print_moveto (context, x, y);
-	gnome_print_lineto (context, x+w, y);
-	gnome_print_lineto (context, x+w, y-h);
-	gnome_print_lineto (context, x, y-h);
-	gnome_print_lineto (context, x, y);
-	gnome_print_fill (context);
+	cairo_new_path (context);
+	cairo_rectangle (context, x, y, w, h);
+	cairo_fill (context);
 }
 
 static void
-print_cell_background (GnomePrintContext *context,
+print_cell_background_gtk (cairo_t *context,
 		       GnmStyle const *style, int col, int row,
 		       float x, float y, float w, float h)
 {
-	if (gnumeric_background_set_pc (style, context))
-		/* Fill the entire cell (API excludes far pixel) */
-		print_rectangle (context, x, y, w+1, h+1);
-	style_border_print_diag (style, context, x, y, x+w, y-h);
+	if (gnumeric_background_set_gtk (style, context))
+		print_rectangle_gtk (context, x, y, w, h);
+	gnm_style_border_print_diag_gtk (style, context, x, y, x+w, y+h);
 }
 
+
 /**
  * print_merged_range:
  *
@@ -210,7 +168,7 @@ print_cell_background (GnomePrintContext *context,
  * segments that are selected.
  */
 static void
-print_merged_range (GnomePrintContext *context, PangoContext *pcontext,
+print_merged_range_gtk (cairo_t *context, PangoContext *pcontext,
 		    Sheet const *sheet,
 		    double start_x, double start_y,
 		    GnmRange const *view, GnmRange const *range)
@@ -237,7 +195,7 @@ print_merged_range (GnomePrintContext *context, PangoContext *pcontext,
 			view->start.row, range->start.row);
 	if (range->end.row <= (last = view->end.row))
 		last = range->end.row;
-	b -= sheet_row_get_distance_pts (sheet, view->start.row, last+1);
+	b += sheet_row_get_distance_pts (sheet, view->start.row, last+1);
 
 	if (l == r || t == b)
 		return;
@@ -250,9 +208,9 @@ print_merged_range (GnomePrintContext *context, PangoContext *pcontext,
 			style = g_ptr_array_index (style->cond_styles, res);
 	}
 
-	if (gnumeric_background_set_pc (style, context))
+	if (gnumeric_background_set_gtk (style, context))
 		/* Remember api excludes the far pixels */
-		print_rectangle (context, l, t, r-l+1, t-b+1);
+		print_rectangle_gtk (context, l, t, r-l+1, t-b+1);
 
 	if (range->start.col < view->start.col)
 		l -= dir * sheet_col_get_distance_pts (sheet,
@@ -261,10 +219,10 @@ print_merged_range (GnomePrintContext *context, PangoContext *pcontext,
 		r += dir * sheet_col_get_distance_pts (sheet,
 			view->end.col+1, range->end.col+1);
 	if (range->start.row < view->start.row)
-		t += sheet_row_get_distance_pts (sheet,
+		t -= sheet_row_get_distance_pts (sheet,
 			range->start.row, view->start.row);
 	if (view->end.row < range->end.row)
-		b -= sheet_row_get_distance_pts (sheet,
+		b += sheet_row_get_distance_pts (sheet,
 			view->end.row+1, range->end.row+1);
 
 	if (cell != NULL) {
@@ -274,13 +232,13 @@ print_merged_range (GnomePrintContext *context, PangoContext *pcontext,
 			row_calc_spans ((ColRowInfo *)ri, cell->pos.row, sheet);
 
 		if (sheet->text_is_rtl)
-			print_cell (cell, style, context, pcontext,
-				r, t, l - r, t - b, -1.);
+			print_cell_gtk (cell, style, context, pcontext,
+				r, t, l - r, b - t, -1.);
 		else
-			print_cell (cell, style, context, pcontext,
-				l, t, r - l, t - b, -1.);
+			print_cell_gtk (cell, style, context, pcontext,
+				l, t, r - l, b - t, -1.);
 	}
-	style_border_print_diag (style, context, l, t, r, b);
+	gnm_style_border_print_diag_gtk (style, context, l, t, r, b);
 }
 
 static gint
@@ -289,11 +247,12 @@ merged_col_cmp (GnmRange const *a, GnmRange const *b)
 	return a->start.col - b->start.col;
 }
 
+
 void
-print_cell_range (GnomePrintContext *context,
-		  Sheet const *sheet, GnmRange *range,
-		  double base_x, double base_y,
-		  gboolean hide_grid)
+gnm_gtk_print_cell_range (GtkPrintContext *print_context, cairo_t *context,
+			  Sheet const *sheet, GnmRange *range,
+			  double base_x, double base_y,
+			  gboolean hide_grid)
 {
 	ColRowInfo const *ri = NULL, *next_ri = NULL;
 	int const dir = sheet->text_is_rtl ? -1 : 1;
@@ -304,7 +263,7 @@ print_cell_range (GnomePrintContext *context,
 	GnmStyle const **styles;
 	GnmBorder const **borders, **prev_vert;
 	GnmBorder const *none =
-		hide_grid ? NULL : style_border_none ();
+		hide_grid ? NULL : gnm_style_border_none ();
 
 	int n, col, row;
 	double x, y, offset;
@@ -312,14 +271,12 @@ print_cell_range (GnomePrintContext *context,
 	GSList	 *merged_active, *merged_active_seen,
 		 *merged_used, *merged_unused, *ptr, **lag;
 
-	g_return_if_fail (GNOME_IS_PRINT_CONTEXT (context));
 	g_return_if_fail (IS_SHEET (sheet));
 	g_return_if_fail (range != NULL);
 	g_return_if_fail (range->start.col <= range->end.col);
 	g_return_if_fail (range->start.row <= range->end.row);
 
-	pcontext = gnome_print_pango_create_context
-		(gnome_print_pango_get_default_font_map ());
+	pcontext = gtk_print_context_create_pango_context (print_context);
 
 	start_col = range->start.col;
 	start_row = range->start.row;
@@ -342,7 +299,7 @@ print_cell_range (GnomePrintContext *context,
 
 	/* Get ordered list of merged regions */
 	merged_active = merged_active_seen = merged_used = NULL;
-	merged_unused = sheet_merge_get_overlap (sheet,
+	merged_unused = gnm_sheet_merge_get_overlap (sheet,
 		range_init (&view, start_col, start_row, end_col, end_row));
 
 	/*
@@ -417,14 +374,14 @@ print_cell_range (GnomePrintContext *context,
 				} else {
 					ColRowInfo const *ci =
 						sheet_col_get_info (sheet, r->start.col);
-				g_slist_free_1 (tmp);
-				merged_active = g_slist_insert_sorted (merged_active, r,
-							(GCompareFunc)merged_col_cmp);
-				MERGE_DEBUG (r, " : unused -> active\n");
+					g_slist_free_1 (tmp);
+					merged_active = g_slist_insert_sorted (merged_active, r,
+									       (GCompareFunc)merged_col_cmp);
+					MERGE_DEBUG (r, " : unused -> active\n");
 
 				if (ci->visible)
-					print_merged_range (context, pcontext, sheet,
-							    base_x, y, &view, r);
+					print_merged_range_gtk (context, pcontext, sheet,
+								base_x, y, &view, r);
 				}
 			} else {
 				lag = &(ptr->next);
@@ -509,7 +466,7 @@ print_cell_range (GnomePrintContext *context,
 			if (dir < 0)
 				x -= ci->size_pts;
 			style = sr.styles [col];
-			print_cell_background (context, style, col, row, x, y,
+			print_cell_background_gtk (context, style, col, row, x, y,
 					       ci->size_pts, ri->size_pts);
 
 			/* Is this part of a span?
@@ -518,19 +475,23 @@ print_cell_range (GnomePrintContext *context,
 			 * 2) Look in the rows hash table to see if
 			 *    there is a span descriptor.
 			 */
-			if (NULL == ri->spans || NULL == (span = row_span_get (ri, col))) {
+			if (NULL == ri->spans || NULL == (span = row_span_get (ri, col)))
+				{
 
 				/* no need to draw blanks */
 				GnmCell const *cell = sheet_cell_get (sheet, col, row);
-				if (!cell_is_empty (cell))
-					print_cell (cell, style, context, pcontext,
-						x, y, ci->size_pts, ri->size_pts, -1.);
+				if (!gnm_cell_is_empty (cell))
+					print_cell_gtk (cell, style,
+							context, pcontext,
+							x, y,
+							ci->size_pts, ri->size_pts, -1.);
 
 			/* Only draw spaning cells after all the backgrounds
 			 * that we are going to draw have been drawn.  No need
 			 * to draw the edit cell, or blanks.
 			 */
-			} else if (col == span->right || col == end_col) {
+				}
+				else if (col == span->right || col == end_col) {
 				GnmCell const *cell = span->cell;
 				int const start_span_col = span->left;
 				int const end_span_col = span->right;
@@ -568,16 +529,17 @@ print_cell_range (GnomePrintContext *context,
 						real_x -= offset;
 				}
 
-				print_cell (cell, style, context, pcontext,
-					real_x, y, tmp_width, ri->size_pts, center_offset);
+				print_cell_gtk (cell, style, context, pcontext,
+						real_x, y, tmp_width, ri->size_pts,
+						center_offset);
 			} else if (col != span->left)
 				sr.vertical [col] = NULL;
 
 			if (dir > 0)
 			x += ci->size_pts;
 		}
-		style_borders_row_print (prev_vert, &sr,
-					 context, base_x, y, y-ri->size_pts,
+		gnm_style_borders_row_print_gtk (prev_vert, &sr,
+					 context, base_x, y, y+ri->size_pts,
 					 sheet, TRUE, dir);
 
 		/* In case there were hidden merges that trailed off the end */
@@ -602,9 +564,9 @@ print_cell_range (GnomePrintContext *context,
 		sr.bottom = next_sr.top = next_sr.bottom; next_sr.bottom = borders;
 		styles = sr.styles; sr.styles = next_sr.styles; next_sr.styles = styles;
 
-		y -= ri->size_pts;
+		y += ri->size_pts;
 	}
-	style_borders_row_print (prev_vert, &sr,
+	gnm_style_borders_row_print_gtk (prev_vert, &sr,
 				 context, base_x, y, y, sheet, FALSE, dir);
 
 	g_slist_free (merged_used);	   /* merges with bottom in view */
@@ -613,3 +575,5 @@ print_cell_range (GnomePrintContext *context,
 	g_object_unref (pcontext);
 	g_return_if_fail (merged_active == NULL);
 }
+
+
