@@ -41,6 +41,13 @@
 #endif
 
 #define CC2XML(s) ((xmlChar const *)(s))
+#define CXML2C(s) ((char const *)(s))
+
+static inline gboolean
+attr_eq (const xmlChar *a, const char *s)
+{
+	return !strcmp (CXML2C (a), s);
+}
 
 static void
 so_image_view_destroy (SheetObjectView *sov)
@@ -476,8 +483,8 @@ gnm_soi_read_xml_dom (SheetObject *so, char const *typename,
 		xmlFree (type);
 		return FALSE;
 	}
-	soi->type       = g_strdup (type);
-	soi->bytes.len  = gsf_base64_decode_simple (content, strlen (content));
+	soi->type       = g_strdup (CXML2C (type));
+	soi->bytes.len  = gsf_base64_decode_simple (content, strlen (CXML2C (content)));
 	soi->bytes.data = g_memdup (content, soi->bytes.len);
 	xmlFree (type);
 	xmlFree (content);
@@ -490,11 +497,11 @@ content_start (GsfXMLIn *xin, xmlChar const **attrs)
 {
 	SheetObject *so = gnm_xml_in_cur_obj (xin);
 	SheetObjectImage *soi = SHEET_OBJECT_IMAGE (so);
-	xmlChar const *image_type = NULL;
+	char const *image_type = NULL;
 
 	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
-		if (0 == strcmp (attrs[0], "image-type"))
-			image_type = attrs[1];
+		if (attr_eq (attrs[0], "image-type"))
+			image_type = CXML2C (attrs[1]);
 
 	soi->type = g_strdup (image_type);
 }
@@ -566,35 +573,6 @@ gnm_soi_copy (SheetObject *dst, SheetObject const *src)
 	new_soi->crop_right	= soi->crop_right;
 }
 
-#ifdef WITH_GNOME_PRINT
-static void
-gnm_soi_print (SheetObject const *so, GnomePrintContext *ctx,
-			  double width, double height)
-{
-	SheetObjectImage *soi = SHEET_OBJECT_IMAGE (so);
-	GdkPixbuf *pixbuf = soi_get_pixbuf (soi, 1.);
-
-	if (pixbuf != NULL) {
-		const guchar *raw_image = gdk_pixbuf_get_pixels (pixbuf);
-		gint rowstride	= gdk_pixbuf_get_rowstride (pixbuf);
-		gint w	= gdk_pixbuf_get_width  (pixbuf);
-		gint h	= gdk_pixbuf_get_height (pixbuf);
-
-		gnome_print_gsave (ctx);
-		gnome_print_translate (ctx, 0, -height);
-		gnome_print_scale (ctx, width, height);
-
-		if (gdk_pixbuf_get_has_alpha (pixbuf))
-			gnome_print_rgbaimage (ctx, raw_image, w, h, rowstride);
-		else
-			gnome_print_rgbimage (ctx, raw_image, w, h, rowstride);
-
-		g_object_unref (G_OBJECT (pixbuf));
-		gnome_print_grestore (ctx);
-	}
-}
-#endif
-
 static void
 gnm_soi_default_size (SheetObject const *so, double *w, double *h)
 {
@@ -650,12 +628,6 @@ gnm_soi_class_init (GObjectClass *object_class)
 	so_class->prep_sax_parser	= gnm_soi_prep_sax_parser;
 	so_class->copy			= gnm_soi_copy;
 	so_class->user_config		= NULL;
-
-	so_class->print			= NULL;
-#ifdef WITH_GNOME_PRINT
-	so_class->print			= gnm_soi_print;
-#endif
-
 	so_class->default_size		= gnm_soi_default_size;
 	so_class->rubber_band_directly	= TRUE;
 
