@@ -60,6 +60,14 @@
 #include <math.h>
 #include <string.h>
 
+#define CXML2C(s) ((char const *)(s))
+
+static inline gboolean
+attr_eq (const xmlChar *a, const char *s)
+{
+	return !strcmp (CXML2C (a), s);
+}
+
 /****************************************************************************/
 
 static void
@@ -107,7 +115,7 @@ static GSF_CLASS_FULL (SOWidgetFooView, so_widget_foo_view,
 #define IS_SHEET_WIDGET_OBJECT(o)    (G_TYPE_CHECK_INSTANCE_TYPE((o), SHEET_OBJECT_WIDGET_TYPE))
 #define SOW_CLASS(so)	 	     (SHEET_OBJECT_WIDGET_CLASS (G_OBJECT_GET_CLASS(so)))
 
-#define SOW_MAKE_TYPE(n1, n2, fn_config, fn_set_sheet, fn_clear_sheet,			\
+#define SOW_MAKE_TYPE(n1, n2, fn_config, fn_set_sheet, fn_clear_sheet, fn_foreach_dep,	\
 		      fn_copy, fn_read_dom, fn_write_sax, fn_prep_sax_parser,		\
 	              fn_get_property, fn_set_property, class_init_code)		\
 static void										\
@@ -122,6 +130,7 @@ sheet_widget_ ## n1 ## _class_init (GObjectClass *object_class)				\
         so_class->interactive           = TRUE;						\
 	so_class->assign_to_sheet	= fn_set_sheet;					\
 	so_class->remove_from_sheet	= fn_clear_sheet;				\
+	so_class->foreach_dep		= fn_foreach_dep;				\
 	so_class->copy			= fn_copy;					\
 	so_class->read_xml_dom		= fn_read_dom;					\
 	so_class->write_xml_sax		= fn_write_sax;					\
@@ -153,7 +162,7 @@ sax_write_dep (GsfXMLOut *output, GnmDependent const *dep, char const *id)
 		GnmParsePos pos;
 		char *val = gnm_expr_top_as_string (dep->texpr,
 			parse_pos_init_sheet (&pos, dep->sheet),
-			gnm_expr_conventions_default);
+			gnm_conventions_default);
 		gsf_xml_out_add_cstr (output, id, val);
 		g_free (val);
 	}
@@ -167,13 +176,13 @@ sax_read_dep (xmlChar const * const *attrs, char const *name,
 	g_return_val_if_fail (attrs[0] != NULL, FALSE);
 	g_return_val_if_fail (attrs[1] != NULL, FALSE);
 
-	if (strcmp (attrs[0], name))
+	if (!attr_eq (attrs[0], name))
 		return FALSE;
 
 	dep->sheet = NULL;
 	if (attrs[1] != NULL && *attrs[1] != '\0') {
 		GnmParsePos pp;
-		dep->texpr = gnm_expr_parse_str_simple (attrs[1],
+		dep->texpr = gnm_expr_parse_str_simple (CXML2C (attrs[1]),
 			parse_pos_init_sheet (&pp, gnm_xml_in_cur_sheet (xin)));
 	} else
 		dep->texpr = NULL;
@@ -308,9 +317,9 @@ sheet_widget_frame_prep_sax_parser (SheetObject *so, GsfXMLIn *xin, xmlChar cons
 {
 	SheetWidgetFrame *swf = SHEET_WIDGET_FRAME (so);
 	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
-		if (0 == strcmp (attrs[0], "Label")) {
+		if (attr_eq (attrs[0], "Label")) {
 			g_free (swf->label);
-			swf->label = g_strdup (attrs[1]);
+			swf->label = g_strdup (CXML2C (attrs[1]));
 		}
 }
 
@@ -472,6 +481,7 @@ SOW_MAKE_TYPE (frame, Frame,
 	       &sheet_widget_frame_user_config,
 	       NULL,
 	       NULL,
+	       NULL,
 	       &sheet_widget_frame_copy,
 	       &sheet_widget_frame_read_xml_dom,
 	       &sheet_widget_frame_write_xml_sax,
@@ -553,7 +563,7 @@ sheet_widget_button_prep_sax_parser (SheetObject *so, GsfXMLIn *xin, xmlChar con
 {
 	SheetWidgetButton *swb = SHEET_WIDGET_BUTTON (so);
 	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
-		if (0 == strcmp (attrs[0], "Label"))
+		if (attr_eq (attrs[0], "Label"))
 			g_object_set (G_OBJECT (swb), "label", attrs[1], NULL);
 }
 
@@ -667,6 +677,7 @@ SOW_MAKE_TYPE (button, Button,
 	       NULL,
 	       NULL,
 	       NULL,
+	       NULL,
 	       sheet_widget_button_copy,
 	       sheet_widget_button_read_xml_dom,
 	       sheet_widget_button_write_xml_sax,
@@ -1039,6 +1050,15 @@ sheet_widget_adjustment_clear_sheet (SheetObject *so)
 	return FALSE;
 }
 
+static void
+sheet_widget_adjustment_foreach_dep (SheetObject *so,
+				     SheetObjectForeachDepFunc func,
+				     gpointer user)
+{
+	SheetWidgetAdjustment *swa = SHEET_WIDGET_ADJUSTMENT (so);
+	func (&swa->dep, so, user);
+}
+
 static void
 sheet_widget_adjustment_write_xml_sax (SheetObject const *so, GsfXMLOut *output)
 {
@@ -1128,13 +1148,14 @@ sheet_widget_adjustment_create_widget (SheetObjectWidget *sow)
 }
 
 SOW_MAKE_TYPE (adjustment, Adjustment,
-	       &sheet_widget_adjustment_user_config,
-	       &sheet_widget_adjustment_set_sheet,
-	       &sheet_widget_adjustment_clear_sheet,
-	       &sheet_widget_adjustment_copy,
-	       &sheet_widget_adjustment_read_xml_dom,
-	       &sheet_widget_adjustment_write_xml_sax,
-	       &sheet_widget_adjustment_prep_sax_parser,
+	       sheet_widget_adjustment_user_config,
+	       sheet_widget_adjustment_set_sheet,
+	       sheet_widget_adjustment_clear_sheet,
+	       sheet_widget_adjustment_foreach_dep,
+	       sheet_widget_adjustment_copy,
+	       sheet_widget_adjustment_read_xml_dom,
+	       sheet_widget_adjustment_write_xml_sax,
+	       sheet_widget_adjustment_prep_sax_parser,
 	       NULL,
 	       NULL,
 	       {})
@@ -1662,6 +1683,15 @@ sheet_widget_checkbox_clear_sheet (SheetObject *so)
 	return FALSE;
 }
 
+static void
+sheet_widget_checkbox_foreach_dep (SheetObject *so,
+				   SheetObjectForeachDepFunc func,
+				   gpointer user)
+{
+	SheetWidgetCheckbox *swc = SHEET_WIDGET_CHECKBOX (so);
+	func (&swc->dep, so, user);
+}
+
 static void
 sheet_widget_checkbox_write_xml_sax (SheetObject const *so, GsfXMLOut *output)
 {
@@ -1678,9 +1708,9 @@ sheet_widget_checkbox_prep_sax_parser (SheetObject *so, GsfXMLIn *xin, xmlChar c
 	SheetWidgetCheckbox *swc = SHEET_WIDGET_CHECKBOX (so);
 
 	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
-		if (0 == strcmp (attrs[0], "Label")) {
+		if (attr_eq (attrs[0], "Label")) {
 			g_free (swc->label);
-			swc->label = g_strdup (attrs[1]);
+			swc->label = g_strdup (CXML2C (attrs[1]));
 		} else if (gnm_xml_attr_int (attrs, "Value", &swc->value))
 			; /* ??? */
 		else if (sax_read_dep (attrs, "Input", &swc->dep, xin))
@@ -1741,15 +1771,16 @@ sheet_widget_checkbox_set_label	(SheetObject *so, char const *str)
 }
 
 SOW_MAKE_TYPE (checkbox, Checkbox,
-	       &sheet_widget_checkbox_user_config,
-	       &sheet_widget_checkbox_set_sheet,
-	       &sheet_widget_checkbox_clear_sheet,
-	       &sheet_widget_checkbox_copy,
-	       &sheet_widget_checkbox_read_xml_dom,
-	       &sheet_widget_checkbox_write_xml_sax,
-	       &sheet_widget_checkbox_prep_sax_parser,
-	       &sheet_widget_checkbox_get_property,
-	       &sheet_widget_checkbox_set_property,
+	       sheet_widget_checkbox_user_config,
+	       sheet_widget_checkbox_set_sheet,
+	       sheet_widget_checkbox_clear_sheet,
+	       sheet_widget_checkbox_foreach_dep,
+	       sheet_widget_checkbox_copy,
+	       sheet_widget_checkbox_read_xml_dom,
+	       sheet_widget_checkbox_write_xml_sax,
+	       sheet_widget_checkbox_prep_sax_parser,
+	       sheet_widget_checkbox_get_property,
+	       sheet_widget_checkbox_set_property,
 	       {
 		       g_object_class_install_property
 			       (object_class, SOC_PROP_TEXT,
@@ -1897,6 +1928,15 @@ sheet_widget_radio_button_clear_sheet (SheetObject *so)
 	return FALSE;
 }
 
+static void
+sheet_widget_radio_button_foreach_dep (SheetObject *so,
+				       SheetObjectForeachDepFunc func,
+				       gpointer user)
+{
+	SheetWidgetRadioButton *swrb = SHEET_WIDGET_RADIO_BUTTON (so);
+	func (&swrb->dep, so, user);
+}
+
 void
 sheet_widget_radio_button_set_label (SheetObject *so, char const *str)
 {
@@ -1968,14 +2008,15 @@ sheet_widget_radio_button_set_property (GObject *obj, guint param_id,
 
 SOW_MAKE_TYPE (radio_button, RadioButton,
 	       NULL,
-	       &sheet_widget_radio_button_set_sheet,
-	       &sheet_widget_radio_button_clear_sheet,
+	       sheet_widget_radio_button_set_sheet,
+	       sheet_widget_radio_button_clear_sheet,
+	       sheet_widget_radio_button_foreach_dep,
 	       NULL,
 	       NULL,
 	       NULL,
 	       NULL,
-	       &sheet_widget_radio_button_get_property,
-	       &sheet_widget_radio_button_set_property,
+	       sheet_widget_radio_button_get_property,
+	       sheet_widget_radio_button_set_property,
 	       {
 		       g_object_class_install_property
 			       (object_class, SOR_PROP_TEXT,
@@ -2092,12 +2133,16 @@ static GnmValue *
 cb_collect (GnmValueIter const *iter, GtkListStore *model)
 {
 	GtkTreeIter list_iter;
-	GOFormat const *fmt = (NULL != iter->cell_iter)
-		? gnm_cell_get_format (iter->cell_iter->cell) : NULL;
-	char *label = format_value (fmt, iter->v, NULL, -1, NULL);
+
 	gtk_list_store_append (model, &list_iter);
-	gtk_list_store_set (model, &list_iter, 0, label, -1);
-	g_free (label);
+	if (NULL != iter->v) {
+		GOFormat const *fmt = (NULL != iter->cell_iter)
+			? gnm_cell_get_format (iter->cell_iter->cell) : NULL;
+		char *label = format_value (fmt, iter->v, NULL, -1, NULL);
+		gtk_list_store_set (model, &list_iter, 0, label, -1);
+		g_free (label);
+	} else
+		gtk_list_store_set (model, &list_iter, 0, "", -1);
 
 	return NULL;
 }
@@ -2162,6 +2207,11 @@ sheet_widget_list_base_finalize (GObject *obj)
 	(*sheet_object_widget_class->finalize) (obj);
 }
 
+static void
+sheet_widget_list_base_user_config (SheetObject *so, SheetControl *sc)
+{
+	dialog_so_list (scg_wbcg (SHEET_CONTROL_GUI (sc)), G_OBJECT (so));
+}
 static gboolean
 sheet_widget_list_base_set_sheet (SheetObject *so, Sheet *sheet)
 {
@@ -2192,6 +2242,16 @@ sheet_widget_list_base_clear_sheet (SheetObject *so)
 	return FALSE;
 }
 
+static void
+sheet_widget_list_base_foreach_dep (SheetObject *so,
+				    SheetObjectForeachDepFunc func,
+				    gpointer user)
+{
+	SheetWidgetListBase *swl = SHEET_WIDGET_LIST_BASE (so);
+	func (&swl->content_dep, so, user);
+	func (&swl->output_dep, so, user);
+}
+
 static void
 sheet_widget_list_base_write_xml_sax (SheetObject const *so, GsfXMLOut *output)
 {
@@ -2233,13 +2293,14 @@ sheet_widget_list_base_create_widget (SheetObjectWidget *sow)
 }
 
 SOW_MAKE_TYPE (list_base, ListBase,
+	       sheet_widget_list_base_user_config,
+	       sheet_widget_list_base_set_sheet,
+	       sheet_widget_list_base_clear_sheet,
+	       sheet_widget_list_base_foreach_dep,
 	       NULL,
-	       &sheet_widget_list_base_set_sheet,
-	       &sheet_widget_list_base_clear_sheet,
-	       NULL,
-	       &sheet_widget_list_base_read_xml_dom,
-	       &sheet_widget_list_base_write_xml_sax,
-	       &sheet_widget_list_base_prep_sax_parser,
+	       sheet_widget_list_base_read_xml_dom,
+	       sheet_widget_list_base_write_xml_sax,
+	       sheet_widget_list_base_prep_sax_parser,
 	       NULL,
 	       NULL,
 	       {
@@ -2274,6 +2335,19 @@ sheet_widget_list_base_set_links (SheetObject *so,
 		list_content_eval (&swl->content_dep); /* populate the list */
 	}
 }
+GnmDependent const *
+sheet_widget_list_base_get_result_dep  (SheetObject const *so)
+{
+	SheetWidgetListBase *swl = SHEET_WIDGET_LIST_BASE (so);
+	return &swl->output_dep;
+}
+
+GnmDependent const *
+sheet_widget_list_base_get_content_dep (SheetObject const *so)
+{
+	SheetWidgetListBase *swl = SHEET_WIDGET_LIST_BASE (so);
+	return &swl->content_dep;
+}
 
 /****************************************************************************/
 #define SHEET_WIDGET_LIST_TYPE	(sheet_widget_list_get_type ())
