@@ -6,10 +6,6 @@
 #include <gtk/gtkselection.h>
 #include <gsf/gsf-output.h>
 
-#ifdef WITH_GNOME_PRINT
-#include <libgnomeprint/gnome-print.h>
-#endif
-
 /***********************************************************
  * Move to goffice during 1.7 */
 
@@ -63,12 +59,6 @@ void	      sheet_object_clear_sheet	 (SheetObject *so);
 SheetObject  *sheet_object_dup		 (SheetObject const *so);
 gboolean      sheet_object_can_print	 (SheetObject const *so);
 
-#ifdef WITH_GNOME_PRINT
-void          sheet_object_print	 (SheetObject const *so,
-					  GnomePrintContext *ctx,
-					  double width, double height);
-#endif
-
 void	     sheet_object_get_editor	 (SheetObject *so, SheetControl *sc);
 void	     sheet_object_populate_menu  (SheetObject *so, GPtrArray *actions);
 
@@ -133,7 +123,7 @@ void sheet_object_write_object 	(SheetObject const *so,
 				 GsfOutput *output, GError **err);
 
 /* cairo rendering */
-void sheet_object_draw_cairo (SheetObject const *so, gpointer *data);
+void sheet_object_draw_cairo (SheetObject const *so, gpointer *data, gboolean rtl);
 
 /* management routine to register all the builtin object types */
 void sheet_objects_init (void);
