@@ -28,6 +28,9 @@
 #include "numbers.h"
 
 
+#define SOLVER_MAX_TIME_ERR _("The maximum time exceeded. The optimal value could not be found in given time.")
+
+
 typedef enum {
 	SolverLPModel, SolverQPModel, SolverNLPModel
 } SolverModelType;
@@ -109,7 +112,7 @@ typedef gpointer SolverProgram;
  * Solver's API for LP solving algorithms
  */
 typedef SolverProgram
-        (solver_init_fn)                (const SolverParameters *param);
+        (solver_init_fn)                (SolverParameters const *param);
 typedef void
         (solver_remove_fn)              (SolverProgram p);
 typedef void
@@ -148,7 +151,7 @@ typedef int
 
 
 typedef struct {
-        const char                    *name;
+        char const                    *name;
         solver_init_fn                *init_fn;
         solver_remove_fn              *remove_fn;
         solver_lp_set_obj_fn          *set_obj_fn;
@@ -241,7 +244,7 @@ void             solver_param_destroy  (SolverParameters *);
 
 /* Creates a copy of the Solver's data structures when a sheet is
  * duplicated. */
-SolverParameters *solver_lp_copy       (const SolverParameters *src_param,
+SolverParameters *solver_lp_copy       (SolverParameters const *src_param,
 					Sheet *new_sheet);
 
 /* Frees the data structure allocated for the results. */
@@ -249,7 +252,7 @@ void             solver_results_free   (SolverResults *res);
 
 /* Returns a pointer to the target cell of the model attached to the
  * given sheet. */
-GnmCell		*solver_get_target_cell (Sheet *sheet);
+GnmCell      	*solver_get_target_cell (Sheet *sheet);
 
 /* Returns a pointer to a input variable cell. */
 GnmCell		*solver_get_input_var (SolverResults *res, int n);
