@@ -21,11 +21,12 @@
  */
 
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include "gnumeric.h"
 #include "stf.h"
 
 #include <goffice/app/file.h>
+#include "cell.h"
 #include "sheet.h"
 #include "sheet-view.h"
 #include "sheet-style.h"
@@ -34,7 +35,7 @@
 #include <goffice/app/io-context-priv.h>
 #include <goffice/utils/go-glib-extras.h>
 #include "command-context.h"
-#include "workbook-control.h"
+#include "workbook-control-gui.h"
 #include "workbook-view.h"
 #include "workbook.h"
 #include "dialog-stf.h"
@@ -224,19 +225,18 @@ stf_read_workbook (GOFileOpener const *fo,  gchar const *enc,
 }
 
 static GnmValue *
-cb_get_content (Sheet *sheet, int col, int row,
-		GnmCell *cell, GsfOutput *buf)
+cb_get_content (GnmCellIter const *iter, GsfOutput *buf)
 {
-	if (cell != NULL) {
+	GnmCell *cell;
+
+	if (NULL != (cell = iter->cell)) {
 		char *tmp;
-		if (cell_has_expr (cell)) {
-			GnmParsePos pp;
+		if (gnm_cell_has_expr (cell))
 			tmp = gnm_expr_top_as_string (cell->base.texpr,
-				parse_pos_init_cell (&pp, cell),
-				gnm_expr_conventions_default);
-		} else if (VALUE_FMT (cell->value) != NULL)
+				&iter->pp, iter->pp.sheet->convs);
+		else if (VALUE_FMT (cell->value) != NULL)
 			tmp = format_value (NULL, cell->value, NULL, -1,
-				workbook_date_conv (sheet->workbook));
+				workbook_date_conv (iter->pp.wb));
 		else
 			tmp = value_get_as_string (cell->value);
 
@@ -273,8 +273,7 @@ stf_text_to_columns (WorkbookControl *wbc, GOCmdContext *cc)
 		return;
 	if (range_width	(src) > 1) {
 		go_cmd_context_error (cc, g_error_new (go_error_invalid (), 0,
-			_("Only 1 one column of <b>input</b> data can be parsed at a time, not %d"),
-			range_width (src)));
+			_("Only one column of <b>input</b> data can be parsed at a time")));
 		return;
 	}
 
@@ -416,7 +415,7 @@ stf_write_workbook (GOFileSaver const *fs, IOContext *context,
 	}
 
 	g_object_set (G_OBJECT (result), "sink", output, NULL);
-	if (stf_export (result) == FALSE)
+	if (gnm_stf_export (result) == FALSE)
 		go_cmd_context_error_import (GO_CMD_CONTEXT (context),
 			_("Error while trying to export file as text"));
 	g_object_unref (result);
@@ -431,10 +430,10 @@ stf_write_csv (GOFileSaver const *fs, IOContext *context,
 		 "sink", output,
 		 "quoting-triggers", ", \t\n\"",
 		 NULL);
-	stf_export_options_sheet_list_add (config,
+	gnm_stf_export_options_sheet_list_add (config,
 					   wb_view_cur_sheet (wbv));
 
-	if (stf_export (config) == FALSE)
+	if (gnm_stf_export (config) == FALSE)
 		go_cmd_context_error_import (GO_CMD_CONTEXT (context),
 			_("Error while trying to write CSV file"));
 
@@ -451,9 +450,9 @@ csv_tsv_probe (GOFileOpener const *fo, GsfInput *input, FileProbeLevel pl)
 	if (pl == FILE_PROBE_CONTENT) {
 		guint8 const *header;
 		gsf_off_t i;
-		const char *enc = NULL;
+		char const *enc = NULL;
 		char *header_utf8;
-		const char *p;
+		char const *p;
 		int try;
 		gboolean ok = TRUE;
 
