@@ -42,6 +42,7 @@ GnmExprSharer *workbook_share_expressions (Workbook *wb, gboolean freeit);
 /* IO Routines */
 gboolean       workbook_set_saveinfo	(Workbook *wb,
 					 FileFormatLevel, GOFileSaver *);
+void           workbook_update_history  (Workbook *wb);
 GOFileSaver *workbook_get_file_saver	(Workbook *wb);
 
 /* See also sheet_cell_foreach_range */
@@ -66,7 +67,7 @@ void     workbook_iteration_tolerance	 (Workbook *wb, double tolerance);
 GODateConventions const *workbook_date_conv (Workbook const *wb);
 gboolean workbook_set_1904 (Workbook *wb, gboolean flag);
 
-void workbook_attach_view (Workbook *wb, WorkbookView *wbv);
+void workbook_attach_view (WorkbookView *wbv);
 void workbook_detach_view (WorkbookView *wbv);
 
 WorkbookSheetState *workbook_sheet_state_new (Workbook const *wb);
