@@ -87,10 +87,10 @@
 
 /* ------------------------------------------------------------------------- */
 
-static GnmExprConventions *
-xml_io_conventions (void)
+static GnmConventions *
+gnm_xml_conventions_new (void)
 {
-	GnmExprConventions *conv = gnm_expr_conventions_new ();
+	GnmConventions *conv = gnm_conventions_new ();
 
 	conv->decimal_sep_dot		= TRUE;
 	conv->output.range_ref		= gnm_1_0_rangeref_as_string;
@@ -127,7 +127,7 @@ xml_parse_ctx_new (xmlDocPtr     doc,
 	ctxt->shared_exprs = g_ptr_array_new ();
 	ctxt->wb_view      = wb_view;
 	ctxt->wb	   = (wb_view != NULL) ? wb_view_get_workbook (wb_view) : NULL;
-	ctxt->exprconv     = xml_io_conventions ();
+	ctxt->convs        = gnm_xml_conventions_new ();
 
 	return ctxt;
 }
@@ -139,7 +139,7 @@ xml_parse_ctx_destroy (XmlParseContext *ctxt)
 
 	g_hash_table_destroy (ctxt->expr_map);
 	g_ptr_array_free (ctxt->shared_exprs, TRUE);
-	gnm_expr_conventions_free (ctxt->exprconv);
+	gnm_conventions_free (ctxt->convs);
 
 	g_free (ctxt);
 }
@@ -241,29 +241,20 @@ xml_node_get_cellpos (xmlNodePtr node, char const *name, GnmCellPos *val)
 }
 
 static void
-xml_node_get_print_unit (xmlNodePtr node, PrintUnit * const pu)
+xml_node_get_print_margin (xmlNodePtr node, double *points, GtkUnit *unit)
 {
 	gchar       *txt;
 
-	g_return_if_fail (pu != NULL);
-	g_return_if_fail (node != NULL);
-
-	xml_node_get_double (node, "Points", &pu->points);
-	txt = (gchar *)xmlGetProp  (node, CC2XML ("PrefUnit"));
-	if (txt) {
-#if 0
-		pu->desired_display = unit_name_to_unit (txt);
-#endif
-		xmlFree (txt);
-	}
-}
-
-static void
-xml_node_get_print_margin (xmlNodePtr node, double *points)
-{
 	g_return_if_fail (node != NULL);
+	g_return_if_fail (points != NULL);
+	g_return_if_fail (unit != NULL);
 
 	xml_node_get_double (node, "Points", points);
+	txt = (gchar *)xmlGetProp  (node, CC2XML ("PrefUnit"));
+	if (txt) {
+		*unit = unit_name_to_unit (txt);
+		xmlFree (txt);
+	}
 }
 
 static gboolean
@@ -395,12 +386,12 @@ xml_read_names (XmlParseContext *ctxt, xmlNodePtr tree,
 		parse_error_init (&perr);
 		texpr = gnm_expr_parse_str (CXML2C (expr_str), &pp,
 					   GNM_EXPR_PARSE_DEFAULT,
-					   ctxt->exprconv, &perr);
+					   ctxt->convs, &perr);
 		/* See http://bugzilla.gnome.org/show_bug.cgi?id=317427 */
 		if (!texpr)
 			texpr = gnm_expr_parse_str (CXML2C (expr_str), &pp,
 						   GNM_EXPR_PARSE_DEFAULT,
-						   gnm_expr_conventions_default,
+						   gnm_conventions_default,
 						   NULL);
 
 		if (texpr != NULL) {
@@ -525,17 +516,23 @@ xml_read_print_margins (XmlParseContext *ctxt, xmlNodePtr tree)
 	g_return_if_fail (pi != NULL);
 
 	if ((child = e_xml_get_child_by_name (tree, CC2XML ("top"))))
-		xml_node_get_print_unit (child, &pi->margin.top);
+		xml_node_get_print_margin (child, &pi->margin.top,
+					   &pi->desired_display.header);
 	if ((child = e_xml_get_child_by_name (tree, CC2XML ("bottom"))))
-		xml_node_get_print_unit (child, &pi->margin.bottom);
+		xml_node_get_print_margin (child, &pi->margin.bottom,
+					   &pi->desired_display.footer);
 	if ((child = e_xml_get_child_by_name (tree, CC2XML ("left"))))
-		xml_node_get_print_margin (child, &left);
+		xml_node_get_print_margin (child, &left,
+					   &pi->desired_display.left);
 	if ((child = e_xml_get_child_by_name (tree, CC2XML ("right"))))
-		xml_node_get_print_margin (child, &right);
+		xml_node_get_print_margin (child, &right,
+					   &pi->desired_display.right);
 	if ((child = e_xml_get_child_by_name (tree, CC2XML ("header"))))
-		xml_node_get_print_margin (child, &header);
+		xml_node_get_print_margin (child, &header,
+					   &pi->desired_display.top);
 	if ((child = e_xml_get_child_by_name (tree, CC2XML ("footer"))))
-		xml_node_get_print_margin (child, &footer);
+		xml_node_get_print_margin (child, &footer,
+					   &pi->desired_display.bottom);
 
 	print_info_set_margins (pi, header, footer, left, right);
 }
@@ -659,8 +656,11 @@ xml_read_print_info (XmlParseContext *ctxt, xmlNodePtr tree)
 
 	if ((child = e_xml_get_child_by_name (tree, CC2XML ("orientation")))) {
 		xmlChar *txt = xmlNodeGetContent (child);
-		/* this was once an enum */
-		pi->portrait_orientation = !strcmp (CXML2C (txt), "portrait");
+#warning TODO: we should also handle inversion
+		print_info_set_paper_orientation (pi,
+						  strcmp (CXML2C (txt), "portrait")
+						  ? GTK_PAGE_ORIENTATION_LANDSCAPE
+						  : GTK_PAGE_ORIENTATION_PORTRAIT);
 		xmlFree (txt);
 	}
 
@@ -888,7 +888,7 @@ xml_read_style (XmlParseContext *ctxt, xmlNodePtr tree, gboolean leave_empty)
 				texpr0 = gnm_expr_parse_str
 					(CXML2C (content), &pp,
 					 GNM_EXPR_PARSE_DEFAULT,
-					 ctxt->exprconv, NULL);
+					 ctxt->convs, NULL);
 				xmlFree (content);
 			}
 			if (NULL != (e_node = e_xml_get_child_by_name (child, CC2XML ("Expression1"))) &&
@@ -896,7 +896,7 @@ xml_read_style (XmlParseContext *ctxt, xmlNodePtr tree, gboolean leave_empty)
 				texpr1 = gnm_expr_parse_str
 					(CXML2C (content), &pp,
 					 GNM_EXPR_PARSE_DEFAULT,
-					 ctxt->exprconv, NULL);
+					 ctxt->convs, NULL);
 				xmlFree (content);
 			}
 
@@ -936,7 +936,7 @@ xml_read_style (XmlParseContext *ctxt, xmlNodePtr tree, gboolean leave_empty)
 				cond.texpr[0] = gnm_expr_parse_str
 					(CXML2C (content), &pp,
 					 GNM_EXPR_PARSE_DEFAULT,
-					 ctxt->exprconv, NULL);
+					 ctxt->convs, NULL);
 				xmlFree (content);
 			} else
 				cond.texpr[0] = NULL;
@@ -945,7 +945,7 @@ xml_read_style (XmlParseContext *ctxt, xmlNodePtr tree, gboolean leave_empty)
 				cond.texpr[1] = gnm_expr_parse_str
 					(CXML2C (content), &pp,
 					 GNM_EXPR_PARSE_DEFAULT,
-					 ctxt->exprconv, NULL);
+					 ctxt->convs, NULL);
 				xmlFree (content);
 			} else
 				cond.texpr[1] = NULL;
@@ -1028,7 +1028,7 @@ xml_cell_set_array_expr (XmlParseContext *ctxt,
 		gnm_expr_parse_str (text,
 				    parse_pos_init_cell (&pp, cell),
 				    GNM_EXPR_PARSE_DEFAULT,
-				    ctxt->exprconv, NULL);
+				    ctxt->convs, NULL);
 
 	g_return_if_fail (texpr != NULL);
 	gnm_cell_set_array_formula (cell->base.sheet,
@@ -1244,7 +1244,7 @@ xml_read_cell (XmlParseContext *ctxt, xmlNodePtr tree)
 					texpr = gnm_expr_parse_str (expr_start,
 								    parse_pos_init_cell (&pos, cell),
 								    GNM_EXPR_PARSE_DEFAULT,
-								    ctxt->exprconv, NULL);
+								    ctxt->convs, NULL);
 				}
 				if (texpr != NULL) {
 					gnm_cell_set_expr (cell, texpr);
@@ -2009,7 +2009,7 @@ xml_read_clipboard_cell (XmlParseContext *ctxt, xmlNodePtr tree,
 			cc->texpr =
 				gnm_expr_parse_str (CXML2C (content), &pp,
 						    GNM_EXPR_PARSE_DEFAULT,
-						    ctxt->exprconv, NULL);
+						    ctxt->convs, NULL);
 
 			g_return_if_fail (cc->texpr != NULL);
 #warning TODO : arrays
