@@ -68,7 +68,7 @@ typedef struct {
 	WorkbookView const *wb_view;	/* View for the new workbook */
 	Workbook const	   *wb;		/* The new workbook */
 	Sheet const 	   *sheet;
-	GnmExprConventions *exprconv;
+	GnmConventions	   *convs;
 	GHashTable	   *expr_map;
 
 	GsfXMLOut *output;
@@ -175,7 +175,7 @@ xml_write_name (GnmOutputXML *state, GnmNamedExpr *nexpr)
 	gsf_xml_out_start_element (state->output, GNM "Name");
 	gsf_xml_out_simple_element (state->output, GNM "name",
 		nexpr->name->str);
-	expr_str = expr_name_as_string (nexpr, NULL, state->exprconv);
+	expr_str = expr_name_as_string (nexpr, NULL, state->convs);
 	gsf_xml_out_simple_element (state->output, GNM "value", expr_str);
 	g_free (expr_str);
 	gsf_xml_out_simple_element (state->output, GNM "position",
@@ -213,23 +213,12 @@ xml_write_geometry (GnmOutputXML *state)
 
 static void
 xml_write_print_unit (GnmOutputXML *state, char const *name,
-		      PrintUnit const *pu)
-{
-	gsf_xml_out_start_element (state->output, name);
-	xml_out_add_points (state->output, "Points", pu->points);
-#if 0
-	gsf_xml_out_add_cstr_unchecked (state->output, "PrefUnit", (const char *)pu->desired_display->abbr);
-#endif
-	gsf_xml_out_end_element (state->output);
-}
-
-static void
-xml_write_print_margin (GnmOutputXML *state, char const *name,
-			double points)
+		      double points, GtkUnit unit)
 {
 	gsf_xml_out_start_element (state->output, name);
 	xml_out_add_points (state->output, "Points", points);
-	gsf_xml_out_add_cstr_unchecked (state->output, "PrefUnit", "Pt");
+	gsf_xml_out_add_cstr_unchecked (state->output, "PrefUnit",
+					unit_to_unit_name (unit));
 	gsf_xml_out_end_element (state->output);
 }
 
@@ -260,24 +249,31 @@ xml_write_print_hf (GnmOutputXML *state, char const *name,
 static void
 xml_write_print_info (GnmOutputXML *state, PrintInformation *pi)
 {
-	char const *paper_name;
+	char  *paper_name;
 	double header;
 	double footer;
 	double left;
 	double right;
+	GtkPageOrientation orient;
 
 	g_return_if_fail (pi != NULL);
 
 	gsf_xml_out_start_element (state->output, GNM "PrintInformation");
 
 	gsf_xml_out_start_element (state->output, GNM "Margins");
-	xml_write_print_unit (state, GNM "top",    &pi->margin.top);
-	xml_write_print_unit (state, GNM "bottom", &pi->margin.bottom);
+	xml_write_print_unit (state, GNM "top",    pi->margin.top,
+			      pi->desired_display.header);
+	xml_write_print_unit (state, GNM "bottom", pi->margin.bottom,
+			      pi->desired_display.footer);
 	print_info_get_margins (pi, &header, &footer, &left, &right);
-	xml_write_print_margin (state, GNM "left", left);
-	xml_write_print_margin (state, GNM "right", right);
-	xml_write_print_margin (state, GNM "header", header);
-	xml_write_print_margin (state, GNM "footer", footer);
+	xml_write_print_unit (state, GNM "left", left,
+			      pi->desired_display.left);
+	xml_write_print_unit (state, GNM "right", right,
+			      pi->desired_display.right);
+	xml_write_print_unit (state, GNM "header", header,
+			      pi->desired_display.top);
+	xml_write_print_unit (state, GNM "footer", footer,
+			      pi->desired_display.bottom);
 	gsf_xml_out_end_element (state->output);
 
 	gsf_xml_out_start_element (state->output, GNM "Scale");
@@ -325,8 +321,13 @@ xml_write_print_info (GnmOutputXML *state, PrintInformation *pi)
 	/* this was once an enum, hence the silly strings */
 	gsf_xml_out_simple_element (state->output, GNM "order",
 		pi->print_across_then_down ? "r_then_d" :"d_then_r");
+
+	orient = print_info_get_paper_orientation (pi);
 	gsf_xml_out_simple_element (state->output, GNM "orientation",
-		pi->portrait_orientation ? "portrait" : "landscape");
+		(orient == GTK_PAGE_ORIENTATION_PORTRAIT
+		 || orient == GTK_PAGE_ORIENTATION_REVERSE_PORTRAIT)
+				    ? "portrait" : "landscape");
+#warning TODO: we should also handle inversion
 
 	xml_write_print_hf (state, GNM "Header", pi->header);
 	xml_write_print_hf (state, GNM "Footer", pi->footer);
@@ -461,12 +462,12 @@ xml_write_gnmstyle (GnmOutputXML *state, GnmStyle const *style)
 
 		parse_pos_init_sheet (&pp, (Sheet *)state->sheet);
 		if (v->texpr[0] != NULL &&
-		    (tmp = gnm_expr_top_as_string (v->texpr[0], &pp, state->exprconv)) != NULL) {
+		    (tmp = gnm_expr_top_as_string (v->texpr[0], &pp, state->convs)) != NULL) {
 			gsf_xml_out_simple_element (state->output, GNM "Expression0", tmp);
 			g_free (tmp);
 		}
 		if (v->texpr[1] != NULL &&
-		    (tmp = gnm_expr_top_as_string (v->texpr[1], &pp, state->exprconv)) != NULL) {
+		    (tmp = gnm_expr_top_as_string (v->texpr[1], &pp, state->convs)) != NULL) {
 			gsf_xml_out_simple_element (state->output, GNM "Expression1", tmp);
 			g_free (tmp);
 		}
@@ -494,12 +495,12 @@ xml_write_gnmstyle (GnmOutputXML *state, GnmStyle const *style)
 				gsf_xml_out_add_int (state->output, "Operator", cond->op);
 				parse_pos_init_sheet (&pp, (Sheet *)state->sheet);
 				if (cond->texpr[0] != NULL &&
-				    (tmp = gnm_expr_top_as_string (cond->texpr[0], &pp, state->exprconv)) != NULL) {
+				    (tmp = gnm_expr_top_as_string (cond->texpr[0], &pp, state->convs)) != NULL) {
 					gsf_xml_out_simple_element (state->output, GNM "Expression0", tmp);
 					g_free (tmp);
 				}
 				if (cond->texpr[1] != NULL &&
-				    (tmp = gnm_expr_top_as_string (cond->texpr[1], &pp, state->exprconv)) != NULL) {
+				    (tmp = gnm_expr_top_as_string (cond->texpr[1], &pp, state->convs)) != NULL) {
 					gsf_xml_out_simple_element (state->output, GNM "Expression1", tmp);
 					g_free (tmp);
 				}
@@ -729,13 +730,18 @@ xml_write_cell_and_position (GnmOutputXML *state,
 					const char *fmt = go_format_as_XL (VALUE_FMT (val));
 					gsf_xml_out_add_cstr (state->output, "ValueFormat", fmt);
 				}
-				value_get_as_gstring (val, str, state->exprconv);
+				value_get_as_gstring (val, str, state->convs);
 			} else {
 				g_warning ("%s has no value ?", cellpos_as_string (&pp->eval));
 			}
 		} else {
+			GnmConventionsOut out;
+			out.accum = str;
+			out.pp    = pp;
+			out.convs = state->convs;
+
 			g_string_append_c (str, '=');
-			gnm_expr_top_as_gstring (str, texpr, pp, state->exprconv);
+			gnm_expr_top_as_gstring (texpr, &out);
 		}
 
 		gsf_xml_out_add_cstr (state->output, NULL, str->str);
@@ -808,7 +814,7 @@ xml_write_filter_expr (GnmOutputXML *state,
 	};
 
 	GString *text = g_string_new (NULL);
-	value_get_as_gstring (cond->value[i], text, state->exprconv);
+	value_get_as_gstring (cond->value[i], text, state->convs);
 	gsf_xml_out_add_cstr_unchecked (state->output,
 		filter_expr_attrs[i].op, filter_cond_name [cond->op[i]]);
 	gsf_xml_out_add_int (state->output,
@@ -1143,10 +1149,10 @@ xml_write_calculation (GnmOutputXML *state)
 	gsf_xml_out_end_element (state->output); /* </gnm:Calculation> */
 }
 
-static GnmExprConventions *
+static GnmConventions *
 xml_io_conventions (void)
 {
-	GnmExprConventions *res = gnm_expr_conventions_new ();
+	GnmConventions *res = gnm_conventions_new ();
 
 	res->decimal_sep_dot		= TRUE;
 	res->input.range_ref		= rangeref_parse;
@@ -1187,7 +1193,7 @@ gnm_xml_file_save (GOFileSaver const *fs, IOContext *io_context,
 	state.wb	= wb_view_get_workbook (wb_view);
 	state.sheet	= NULL;
 	state.output	= gsf_xml_out_new (output);
-	state.exprconv	= xml_io_conventions ();
+	state.convs	= xml_io_conventions ();
 	state.expr_map  = g_hash_table_new (g_direct_hash, g_direct_equal);
 
 	locale = gnm_push_C_locale ();
@@ -1216,7 +1222,7 @@ gnm_xml_file_save (GOFileSaver const *fs, IOContext *io_context,
 	gnm_pop_C_locale (locale);
 
 	g_hash_table_destroy (state.expr_map);
-	gnm_expr_conventions_free (state.exprconv);
+	gnm_conventions_free (state.convs);
 	g_object_unref (G_OBJECT (state.output));
 
 	if (gzout) {
@@ -1267,7 +1273,7 @@ gnm_cellregion_to_xml (GnmCellRegion const *cr)
 	state.state.wb	= NULL;
 	state.state.sheet	= NULL;
 	state.state.output	= gsf_xml_out_new (buf);
-	state.state.exprconv	= xml_io_conventions ();
+	state.state.convs	= xml_io_conventions ();
 	state.state.expr_map  = g_hash_table_new (g_direct_hash, g_direct_equal);
 
 	locale = gnm_push_C_locale ();
@@ -1318,7 +1324,7 @@ gnm_cellregion_to_xml (GnmCellRegion const *cr)
 	gnm_pop_C_locale (locale);
 
 	g_hash_table_destroy (state.state.expr_map);
-	gnm_expr_conventions_free (state.state.exprconv);
+	gnm_conventions_free (state.state.convs);
 	g_object_unref (G_OBJECT (state.state.output));
 
 	gsf_output_close (buf);
